# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from .wl_surface import WlSurface


class WlTouch(Interface):
    """Touchscreen input device

    The :class:`WlTouch` interface represents a touchscreen associated with a
    seat.

    Touch interactions can consist of one or more contacts. For each contact, a
    series of events is generated, starting with a down event, followed by zero
    or more motion events, and ending with an up event. Events relating to the
    same contact point can be identified by the ID of the sequence.
    """

    name = "wl_touch"
    version = 7


class WlTouchProxy(Proxy):
    interface = WlTouch

    @WlTouch.request(version=3)
    def release(self):
        """Release the touch object
        """
        self._marshal(0)
        self._destroy()


class WlTouchResource(Resource):
    interface = WlTouch

    @WlTouch.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Object, interface=WlSurface),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Fixed),
        Argument(ArgumentType.Fixed),
    )
    def down(self, serial, time, surface, id, x, y):
        """Touch down event and beginning of a touch sequence

        A new touch point has appeared on the surface. This touch point is
        assigned a unique ID. Future events from this touch point reference
        this ID. The ID ceases to be valid after a touch up event and may be
        reused in the future.

        :param serial:
            serial number of the touch down event
        :type serial:
            `ArgumentType.Uint`
        :param time:
            timestamp with millisecond granularity
        :type time:
            `ArgumentType.Uint`
        :param surface:
            surface touched
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :param id:
            the unique ID of this touch point
        :type id:
            `ArgumentType.Int`
        :param x:
            surface-local x coordinate
        :type x:
            `ArgumentType.Fixed`
        :param y:
            surface-local y coordinate
        :type y:
            `ArgumentType.Fixed`
        """
        self._post_event(0, serial, time, surface, id, x, y)

    @WlTouch.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Int),
    )
    def up(self, serial, time, id):
        """End of a touch event sequence

        The touch point has disappeared. No further events will be sent for
        this touch point and the touch point's ID is released and may be reused
        in a future touch down event.

        :param serial:
            serial number of the touch up event
        :type serial:
            `ArgumentType.Uint`
        :param time:
            timestamp with millisecond granularity
        :type time:
            `ArgumentType.Uint`
        :param id:
            the unique ID of this touch point
        :type id:
            `ArgumentType.Int`
        """
        self._post_event(1, serial, time, id)

    @WlTouch.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Fixed),
        Argument(ArgumentType.Fixed),
    )
    def motion(self, time, id, x, y):
        """Update of touch point coordinates

        A touch point has changed coordinates.

        :param time:
            timestamp with millisecond granularity
        :type time:
            `ArgumentType.Uint`
        :param id:
            the unique ID of this touch point
        :type id:
            `ArgumentType.Int`
        :param x:
            surface-local x coordinate
        :type x:
            `ArgumentType.Fixed`
        :param y:
            surface-local y coordinate
        :type y:
            `ArgumentType.Fixed`
        """
        self._post_event(2, time, id, x, y)

    @WlTouch.event()
    def frame(self):
        """End of touch frame event

        Indicates the end of a set of events that logically belong together. A
        client is expected to accumulate the data in all events within the
        frame before proceeding.

        A :func:`WlTouch.frame()` terminates at least one event but otherwise
        no guarantee is provided about the set of events within a frame. A
        client must assume that any state not updated in a frame is unchanged
        from the previously known state.
        """
        self._post_event(3)

    @WlTouch.event()
    def cancel(self):
        """Touch session cancelled

        Sent if the compositor decides the touch stream is a global gesture. No
        further events are sent to the clients from that particular gesture.
        Touch cancellation applies to all touch points currently active on this
        client's surface. The client is responsible for finalizing the touch
        points, future touch points on this surface may reuse the touch point
        ID.
        """
        self._post_event(4)

    @WlTouch.event(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Fixed),
        Argument(ArgumentType.Fixed),
        version=6,
    )
    def shape(self, id, major, minor):
        """Update shape of touch point

        Sent when a touchpoint has changed its shape.

        This event does not occur on its own. It is sent before a
        :func:`WlTouch.frame()` event and carries the new shape information for
        any previously reported, or new touch points of that frame.

        Other events describing the touch point such as :func:`WlTouch.down()`,
        :func:`WlTouch.motion()` or :func:`WlTouch.orientation()` may be sent
        within the same :func:`WlTouch.frame()`. A client should treat these
        events as a single logical touch point update. The order of
        :func:`WlTouch.shape()`, :func:`WlTouch.orientation()` and
        :func:`WlTouch.motion()` is not guaranteed. A :func:`WlTouch.down()`
        event is guaranteed to occur before the first :func:`WlTouch.shape()`
        event for this touch ID but both events may occur within the same
        :func:`WlTouch.frame()`.

        A touchpoint shape is approximated by an ellipse through the major and
        minor axis length. The major axis length describes the longer diameter
        of the ellipse, while the minor axis length describes the shorter
        diameter. Major and minor are orthogonal and both are specified in
        surface-local coordinates. The center of the ellipse is always at the
        touchpoint location as reported by :func:`WlTouch.down()` or
        :func:`WlTouch.move()`.

        This event is only sent by the compositor if the touch device supports
        shape reports. The client has to make reasonable assumptions about the
        shape if it did not receive this event.

        :param id:
            the unique ID of this touch point
        :type id:
            `ArgumentType.Int`
        :param major:
            length of the major axis in surface-local coordinates
        :type major:
            `ArgumentType.Fixed`
        :param minor:
            length of the minor axis in surface-local coordinates
        :type minor:
            `ArgumentType.Fixed`
        """
        self._post_event(5, id, major, minor)

    @WlTouch.event(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Fixed),
        version=6,
    )
    def orientation(self, id, orientation):
        """Update orientation of touch point

        Sent when a touchpoint has changed its orientation.

        This event does not occur on its own. It is sent before a
        :func:`WlTouch.frame()` event and carries the new shape information for
        any previously reported, or new touch points of that frame.

        Other events describing the touch point such as :func:`WlTouch.down()`,
        :func:`WlTouch.motion()` or :func:`WlTouch.shape()` may be sent within
        the same :func:`WlTouch.frame()`. A client should treat these events as
        a single logical touch point update. The order of
        :func:`WlTouch.shape()`, :func:`WlTouch.orientation()` and
        :func:`WlTouch.motion()` is not guaranteed. A :func:`WlTouch.down()`
        event is guaranteed to occur before the first
        :func:`WlTouch.orientation()` event for this touch ID but both events
        may occur within the same :func:`WlTouch.frame()`.

        The orientation describes the clockwise angle of a touchpoint's major
        axis to the positive surface y-axis and is normalized to the -180 to
        +180 degree range. The granularity of orientation depends on the touch
        device, some devices only support binary rotation values between 0 and
        90 degrees.

        This event is only sent by the compositor if the touch device supports
        orientation reports.

        :param id:
            the unique ID of this touch point
        :type id:
            `ArgumentType.Int`
        :param orientation:
            angle between major axis and positive surface y-axis in degrees
        :type orientation:
            `ArgumentType.Fixed`
        """
        self._post_event(6, id, orientation)


class WlTouchGlobal(Global):
    interface = WlTouch


WlTouch._gen_c()
WlTouch.proxy_class = WlTouchProxy
WlTouch.resource_class = WlTouchResource
WlTouch.global_class = WlTouchGlobal



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import contextlib
import re
import textwrap
from typing import Iterator, List, Mapping, Match, Optional, BinaryIO

HEAD_MSG = "# This file has been autogenerated by the pywayland scanner"

# Match the start of unordered lists
RE_DOC_LIST = re.compile(r"^(?P<list_head>\* |- )")
# Match the interface and function use in the docstrings
RE_DOC = r"(?<![_\w])(?P<interface>{})(?P<func>\.[a-z][_a-z]*(?:\(\))?)?(?=[^\w\*']|$)"
# The base import path for sphinx directives in docstrings
BASE_PATH = "pywayland.protocol"
# the width of the doc string
DOC_WIDTH = 79
# the size of the tab stops
TAB_STOP = 4


class Printer:
    def __init__(
        self,
        protocol: str,
        interface_name: Optional[str] = None,
        interface_imports: Optional[Mapping[str, str]] = None,
    ) -> None:
        """Base level printer object

        Allows for storing of lines to be output from the definition of a
        protocol.  Lines are added by directly calling the printer object.

        :param protocol:
            The name of the protocol that is currently being generated, used
            for determining import resolution.
        :param interface_name:
            The name of the interface that is being generated, used for
            determining import resolution.
        :param interface_imports:
            The map from the interface name to the protocol it is defined in,
            for resolving imports.
        """
        self._level = 0
        self._lines = [HEAD_MSG, ""]
        self._protocol_name = protocol
        self._interface_name = interface_name
        self._interface_imports = interface_imports

        self._re_doc = None
        if interface_imports is not None:
            interface_names = "|".join(sorted(interface_imports, key=len, reverse=True))
            self._re_doc = re.compile(RE_DOC.format(interface_names))

    def __call__(self, new_line: Optional[str] = None) -> None:
        """Add the new line to the printer

        :param new_line:
            The new line to add to the file, should be appropriately wrapped.
        """
        if new_line:
            self._lines.append((" " * TAB_STOP * self._level) + new_line)
        else:
            self._lines.append("")

    def docstring(self, docstring: str) -> None:
        """Add lines as docstrings

        In addition to the operations performed by :meth:`Printer.doc()`, will
        wrap text passed to it to the correct width.

        :param docstring:
            The docstring line to add to the printer.
        """
        docstring = self._parse_doc_line(docstring)

        # create a list of properly indented paragraphs
        paragraphs = []
        for paragraph in docstring.split("\n\n"):
            # try to detect and properly output lists
            doc_list_match = RE_DOC_LIST.search(paragraph.lstrip())

            base_width = DOC_WIDTH - TAB_STOP * self._level
            if doc_list_match is None:
                # not a list, just create the paragraph
                paragraphs.append(
                    textwrap.fill(paragraph, width=base_width, break_long_words=False)
                )
            else:
                # the list items are not always separated by paragraph breaks,
                # so parse each line and see if they denote list items
                start_list = doc_list_match.group("list_head")
                lines = paragraph.split("\n")
                current_list_item = [lines[0].strip()]
                list_items: List[str] = []
                for line in lines[1:]:
                    if line.strip().startswith(start_list):
                        # store the current list item
                        list_items.append(" ".join(current_list_item))
                        # start a new list item
                        current_list_item = [line.strip()]
                    else:
                        current_list_item.append(line.strip())
                # store the last list item
                list_items.append(" ".join(current_list_item))

                # add each list item as a new paragraph
                paragraphs.extend(
                    textwrap.fill(
                        list_item,
                        width=base_width,
                        subsequent_indent=" " * len(start_list),
                        break_long_words=False,
                    )
                    for list_item in list_items
                )

        wrapped = "\n\n".join(paragraphs)
        for line in wrapped.split("\n"):
            self(line)

    def doc(self, new_line: str) -> None:
        """Add lines as docstrings

        Performs additional massaging of strings, replacing references to other
        protocols and protocol methods with the appropriate Sphinx
        cross-reference.

        :param new_line:
            The new line to add to the printer output.
        """
        new_line = self._parse_doc_line(new_line)
        self(new_line)

    def _parse_doc_line(self, line: str) -> str:
        """Parse the given docstring input

        Perform all necessary class and function replacements that match the
        given set of interfaces that are known.
        """
        assert self._re_doc is not None

        line = self._re_doc.sub(self._doc_replace, line)
        return line

    def _doc_replace(self, match: Match) -> str:
        """Perform interface and function name replacement on the given match"""
        if match.group("func") is None:
            return self._doc_class_replace(match)
        return self._doc_funcs_replace(match)

    def _doc_class_replace(self, match: Match) -> str:
        """Build the sphinx doc class import

        :param match:
            The regex match for the given interface class.
        :returns:
            The string corresponding to the sphinx formatted class.
        """
        interface_name = match.group("interface")
        interface_class = "".join(x.capitalize() for x in interface_name.split("_"))

        if interface_name == self._interface_name:
            return ":class:`{}`".format(interface_class)

        if (
            self._interface_imports is not None
            and interface_name in self._interface_imports
        ):
            protocol_path = self._interface_imports[interface_name]
            return ":class:`~{base_path}.{iface}.{class_name}`".format(
                class_name=interface_class,
                base_path=BASE_PATH,
                iface=protocol_path,
            )

        return "`{}`".format(interface_class)

    def _doc_funcs_replace(self, match: Match) -> str:
        """Build the sphinx doc function definition

        :param match:
            The regex match for the given interface and function.
        :returns:
            The string corresponding to the sphinx formatted function.
        """
        interface_name = match.group("interface")
        function_name = match.group("func")
        interface_class = "".join(x.capitalize() for x in interface_name.split("_"))

        if interface_name == self._interface_name:
            return ":func:`{}{}()`".format(interface_class, function_name)

        if (
            self._interface_imports is not None
            and interface_name in self._interface_imports
        ):
            protocol_path = self._interface_imports[interface_name]
            return ":func:`{class_name}{func}() <{base_path}.{iface}.{class_name}{func}>`".format(
                class_name=interface_class,
                func=function_name,
                base_path=BASE_PATH,
                iface=protocol_path,
            )

        return "`{}{}()`".format(interface_class, function_name)

    @contextlib.contextmanager
    def indented(self) -> Iterator[None]:
        """Indent in a level in the context manager block"""
        self._level += 1
        yield
        self._level -= 1

    def write(self, f: BinaryIO) -> None:
        """Write the lines added to the printer out to the given file"""
        for line in self._lines:
            f.write(line.encode())
            f.write(b"\n")



# This file has been autogenerated by the pywayland scanner

# Copyright © 2015-2016 Red Hat Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource


class ZxdgExportedV1(Interface):
    """An exported surface handle

    An xdg_exported object represents an exported reference to a surface. The
    exported surface may be referenced as long as the xdg_exported object not
    destroyed. Destroying the xdg_exported invalidates any relationship the
    importer may have established using xdg_imported.
    """

    name = "zxdg_exported_v1"
    version = 1


class ZxdgExportedV1Proxy(Proxy):
    interface = ZxdgExportedV1

    @ZxdgExportedV1.request()
    def destroy(self):
        """Unexport the exported surface

        Revoke the previously exported surface. This invalidates any
        relationship the importer may have set up using the xdg_imported
        created given the handle sent via xdg_exported.handle.
        """
        self._marshal(0)
        self._destroy()


class ZxdgExportedV1Resource(Resource):
    interface = ZxdgExportedV1

    @ZxdgExportedV1.event(
        Argument(ArgumentType.String),
    )
    def handle(self, handle):
        """The exported surface handle

        The handle event contains the unique handle of this exported surface
        reference. It may be shared with any client, which then can use it to
        import the surface by calling xdg_importer.import. A handle may be used
        to import the surface multiple times.

        :param handle:
            the exported surface handle
        :type handle:
            `ArgumentType.String`
        """
        self._post_event(0, handle)


class ZxdgExportedV1Global(Global):
    interface = ZxdgExportedV1


ZxdgExportedV1._gen_c()
ZxdgExportedV1.proxy_class = ZxdgExportedV1Proxy
ZxdgExportedV1.resource_class = ZxdgExportedV1Resource
ZxdgExportedV1.global_class = ZxdgExportedV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2020 Aleix Pol Gonzalez <aleixpol@kde.org>
# Copyright © 2020 Carlos Garnacho <carlosg@gnome.org>
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlSurface
from .xdg_activation_token_v1 import XdgActivationTokenV1


class XdgActivationV1(Interface):
    """Interface for activating surfaces

    A global interface used for informing the compositor about applications
    being activated or started, or for applications to request to be activated.
    """

    name = "xdg_activation_v1"
    version = 1


class XdgActivationV1Proxy(Proxy):
    interface = XdgActivationV1

    @XdgActivationV1.request()
    def destroy(self):
        """Destroy the xdg_activation object

        Notify the compositor that the xdg_activation object will no longer be
        used.

        The child objects created via this interface are unaffected and should
        be destroyed separately.
        """
        self._marshal(0)
        self._destroy()

    @XdgActivationV1.request(
        Argument(ArgumentType.NewId, interface=XdgActivationTokenV1),
    )
    def get_activation_token(self):
        """Requests a token

        Creates an
        :class:`~pywayland.protocol.xdg_activation_v1.XdgActivationTokenV1`
        object that will provide the initiating client with a unique token for
        this activation. This token should be offered to the clients to be
        activated.

        :returns:
            :class:`~pywayland.protocol.xdg_activation_v1.XdgActivationTokenV1`
        """
        id = self._marshal_constructor(1, XdgActivationTokenV1)
        return id

    @XdgActivationV1.request(
        Argument(ArgumentType.String),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def activate(self, token, surface):
        """Notify new interaction being available

        Requests surface activation. It's up to the compositor to display this
        information as desired, for example by placing the surface above the
        rest.

        The compositor may know who requested this by checking the activation
        token and might decide not to follow through with the activation if
        it's considered unwanted.

        Compositors can ignore unknown activation tokens when an invalid token
        is passed.

        :param token:
            the activation token of the initiating client
        :type token:
            `ArgumentType.String`
        :param surface:
            the :class:`~pywayland.protocol.wayland.WlSurface` to activate
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        """
        self._marshal(2, token, surface)


class XdgActivationV1Resource(Resource):
    interface = XdgActivationV1


class XdgActivationV1Global(Global):
    interface = XdgActivationV1


XdgActivationV1._gen_c()
XdgActivationV1.proxy_class = XdgActivationV1Proxy
XdgActivationV1.resource_class = XdgActivationV1Resource
XdgActivationV1.global_class = XdgActivationV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from .wl_buffer import WlBuffer


class WlShmPool(Interface):
    """A shared memory pool

    The :class:`WlShmPool` object encapsulates a piece of memory shared between
    the compositor and client.  Through the :class:`WlShmPool` object, the
    client can allocate shared memory
    :class:`~pywayland.protocol.wayland.WlBuffer` objects. All objects created
    through the same pool share the same underlying mapped memory. Reusing the
    mapped memory avoids the setup/teardown overhead and is useful when
    interactively resizing a surface or for many small buffers.
    """

    name = "wl_shm_pool"
    version = 1


class WlShmPoolProxy(Proxy):
    interface = WlShmPool

    @WlShmPool.request(
        Argument(ArgumentType.NewId, interface=WlBuffer),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Uint),
    )
    def create_buffer(self, offset, width, height, stride, format):
        """Create a buffer from the pool

        Create a :class:`~pywayland.protocol.wayland.WlBuffer` object from the
        pool.

        The buffer is created offset bytes into the pool and has width and
        height as specified.  The stride argument specifies the number of bytes
        from the beginning of one row to the beginning of the next.  The format
        is the pixel format of the buffer and must be one of those advertised
        through the :func:`WlShm.format()
        <pywayland.protocol.wayland.WlShm.format>` event.

        A buffer will keep a reference to the pool it was created from so it is
        valid to destroy the pool immediately after creating a buffer from it.

        :param offset:
            buffer byte offset within the pool
        :type offset:
            `ArgumentType.Int`
        :param width:
            buffer width, in pixels
        :type width:
            `ArgumentType.Int`
        :param height:
            buffer height, in pixels
        :type height:
            `ArgumentType.Int`
        :param stride:
            number of bytes from the beginning of one row to the beginning of
            the next row
        :type stride:
            `ArgumentType.Int`
        :param format:
            buffer pixel format
        :type format:
            `ArgumentType.Uint`
        :returns:
            :class:`~pywayland.protocol.wayland.WlBuffer` -- buffer to create
        """
        id = self._marshal_constructor(0, WlBuffer, offset, width, height, stride, format)
        return id

    @WlShmPool.request()
    def destroy(self):
        """Destroy the pool

        Destroy the shared memory pool.

        The mmapped memory will be released when all buffers that have been
        created from this pool are gone.
        """
        self._marshal(1)
        self._destroy()

    @WlShmPool.request(
        Argument(ArgumentType.Int),
    )
    def resize(self, size):
        """Change the size of the pool mapping

        This request will cause the server to remap the backing memory for the
        pool from the file descriptor passed when the pool was created, but
        using the new size.  This request can only be used to make the pool
        bigger.

        :param size:
            new size of the pool, in bytes
        :type size:
            `ArgumentType.Int`
        """
        self._marshal(2, size)


class WlShmPoolResource(Resource):
    interface = WlShmPool


class WlShmPoolGlobal(Global):
    interface = WlShmPool


WlShmPool._gen_c()
WlShmPool.proxy_class = WlShmPoolProxy
WlShmPool.resource_class = WlShmPoolResource
WlShmPool.global_class = WlShmPoolGlobal



# This file has been autogenerated by the pywayland scanner

# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlSurface


class ZwpPointerGesturePinchV1(Interface):
    """A pinch gesture object

    A pinch gesture object notifies a client about a multi-finger pinch gesture
    detected on an indirect input device such as a touchpad. The gesture is
    usually initiated by multiple fingers moving towards each other or away
    from each other, or by two or more fingers rotating around a logical center
    of gravity. The precise conditions of when such a gesture is detected are
    implementation-dependent.

    A gesture consists of three stages: begin, update (optional) and end. There
    cannot be multiple simultaneous hold, pinch or swipe gestures on a same
    pointer/seat, how compositors prevent these situations is implementation-
    dependent.

    A gesture may be cancelled by the compositor or the hardware. Clients
    should not consider performing permanent or irreversible actions until the
    end of a gesture has been received.
    """

    name = "zwp_pointer_gesture_pinch_v1"
    version = 2


class ZwpPointerGesturePinchV1Proxy(Proxy):
    interface = ZwpPointerGesturePinchV1

    @ZwpPointerGesturePinchV1.request()
    def destroy(self):
        """Destroy the pinch gesture object
        """
        self._marshal(0)
        self._destroy()


class ZwpPointerGesturePinchV1Resource(Resource):
    interface = ZwpPointerGesturePinchV1

    @ZwpPointerGesturePinchV1.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Object, interface=WlSurface),
        Argument(ArgumentType.Uint),
    )
    def begin(self, serial, time, surface, fingers):
        """Multi-finger pinch begin

        This event is sent when a multi-finger pinch gesture is detected on the
        device.

        :param serial:
        :type serial:
            `ArgumentType.Uint`
        :param time:
            timestamp with millisecond granularity
        :type time:
            `ArgumentType.Uint`
        :param surface:
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :param fingers:
            number of fingers
        :type fingers:
            `ArgumentType.Uint`
        """
        self._post_event(0, serial, time, surface, fingers)

    @ZwpPointerGesturePinchV1.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Fixed),
        Argument(ArgumentType.Fixed),
        Argument(ArgumentType.Fixed),
        Argument(ArgumentType.Fixed),
    )
    def update(self, time, dx, dy, scale, rotation):
        """Multi-finger pinch motion

        This event is sent when a multi-finger pinch gesture changes the
        position of the logical center, the rotation or the relative scale.

        The dx and dy coordinates are relative coordinates in the surface
        coordinate space of the logical center of the gesture.

        The scale factor is an absolute scale compared to the
        pointer_gesture_pinch.begin event, e.g. a scale of 2 means the fingers
        are now twice as far apart as on pointer_gesture_pinch.begin.

        The rotation is the relative angle in degrees clockwise compared to the
        previous pointer_gesture_pinch.begin or pointer_gesture_pinch.update
        event.

        :param time:
            timestamp with millisecond granularity
        :type time:
            `ArgumentType.Uint`
        :param dx:
            delta x coordinate in surface coordinate space
        :type dx:
            `ArgumentType.Fixed`
        :param dy:
            delta y coordinate in surface coordinate space
        :type dy:
            `ArgumentType.Fixed`
        :param scale:
            scale relative to the initial finger position
        :type scale:
            `ArgumentType.Fixed`
        :param rotation:
            angle in degrees cw relative to the previous event
        :type rotation:
            `ArgumentType.Fixed`
        """
        self._post_event(1, time, dx, dy, scale, rotation)

    @ZwpPointerGesturePinchV1.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Int),
    )
    def end(self, serial, time, cancelled):
        """Multi-finger pinch end

        This event is sent when a multi-finger pinch gesture ceases to be
        valid. This may happen when one or more fingers are lifted or the
        gesture is cancelled.

        When a gesture is cancelled, the client should undo state changes
        caused by this gesture. What causes a gesture to be cancelled is
        implementation-dependent.

        :param serial:
        :type serial:
            `ArgumentType.Uint`
        :param time:
            timestamp with millisecond granularity
        :type time:
            `ArgumentType.Uint`
        :param cancelled:
            1 if the gesture was cancelled, 0 otherwise
        :type cancelled:
            `ArgumentType.Int`
        """
        self._post_event(2, serial, time, cancelled)


class ZwpPointerGesturePinchV1Global(Global):
    interface = ZwpPointerGesturePinchV1


ZwpPointerGesturePinchV1._gen_c()
ZwpPointerGesturePinchV1.proxy_class = ZwpPointerGesturePinchV1Proxy
ZwpPointerGesturePinchV1.resource_class = ZwpPointerGesturePinchV1Resource
ZwpPointerGesturePinchV1.global_class = ZwpPointerGesturePinchV1Global



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from dataclasses import dataclass
from typing import Iterable, Optional
import xml.etree.ElementTree as ET

from .argument import Argument, ArgumentType
from .description import Description
from .method import Method
from .printer import Printer

# For 'new_id' types with no 'interface'
NO_IFACE = "interface"
NO_IFACE_VERSION = "version"


@dataclass(frozen=True)
class Request(Method):
    """Scanner for request objects (client-side method)

    Required attributes: `name`

    Optional attributes: `type` and `since`

    Child elements: `description` and `arg`
    """

    method_type = "request"

    type: Optional[str]

    @classmethod
    def parse(cls, element: ET.Element) -> "Request":
        name = cls.parse_attribute(element, "name")
        if name in ("global", "import"):
            name += "_"

        return cls(
            name=name,
            since=cls.parse_optional_attribute(element, "since"),
            description=cls.parse_optional_child(element, Description, "description"),
            arg=cls.parse_repeated_child(element, Argument, "arg"),
            type=cls.parse_optional_attribute(element, "type"),
        )

    @property
    def new_id(self) -> Optional[Argument]:
        for arg in self.arg:
            if arg.type == ArgumentType.NewId:
                return arg
        return None

    @property
    def method_args(self) -> Iterable[str]:
        """Generator of the arguments to the method

        The `new_id` args are generated in marshaling the args, they do not
        appear in the args of the method.
        """
        for arg in self.arg:
            if arg.type == ArgumentType.NewId:
                # An interface of known type is created for us
                if arg.interface:
                    continue
                # A `new_id` with no interface, c.f. wl_registry_bind
                # Need a string (interface name) and int (interface version)
                yield NO_IFACE
                yield NO_IFACE_VERSION
            else:
                yield arg.name

    @property
    def marshal_args(self) -> Iterable[str]:
        """Arguments sent to `._marshal`"""
        for arg in self.arg:
            if arg.type == ArgumentType.NewId:
                if not arg.interface:
                    yield "{}.name".format(NO_IFACE)
                    yield NO_IFACE_VERSION
            else:
                yield arg.name

    def output_doc_params(self, printer: Printer) -> None:
        """Aguments documented as parameters

        Anything that is not a `new_id` is
        """
        ret = None
        for arg in self.arg:
            if arg.type == ArgumentType.NewId:
                ret = arg
                if arg.interface:
                    continue
                printer(f":param {NO_IFACE}:")
                printer("    Interface name")
                printer(f":type {NO_IFACE}:")
                printer("    `string`")
                printer(f":param {NO_IFACE_VERSION}:")
                printer("    Interface version")
                printer(f":type {NO_IFACE_VERSION}:")
                printer("    `int`")
            else:
                arg.output_doc_param(printer)
        if ret is not None:
            ret.output_doc_ret(printer)

    def output_doc_ret(self, printer: Printer) -> None:
        """Aguments documented as return values

        Arguments of type `new_id` are returned from requests.
        """
        for arg in self.arg:
            if arg.type == ArgumentType.NewId:
                arg.output_doc_ret(printer)

    def output_body(self, printer: Printer, opcode: int) -> None:
        """Output the body of the request to the printer"""
        if self.new_id:
            if self.new_id.interface:
                id_class = self.new_id.interface_class
            else:
                id_class = NO_IFACE
            args = ", ".join([str(opcode), id_class] + list(self.marshal_args))
            printer("{} = self._marshal_constructor({})".format(self.new_id.name, args))
            printer("return {}".format(self.new_id.name))
        else:
            args = ", ".join([str(opcode)] + list(self.marshal_args))
            printer("self._marshal({})".format(args))
            if self.type == "destructor":
                printer("self._destroy()")



# This file has been autogenerated by the pywayland scanner

# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlPointer
from .zwp_pointer_gesture_hold_v1 import ZwpPointerGestureHoldV1
from .zwp_pointer_gesture_pinch_v1 import ZwpPointerGesturePinchV1
from .zwp_pointer_gesture_swipe_v1 import ZwpPointerGestureSwipeV1


class ZwpPointerGesturesV1(Interface):
    """Touchpad gestures

    A global interface to provide semantic touchpad gestures for a given
    pointer.

    Three gestures are currently supported: swipe, pinch, and hold. Pinch and
    swipe gestures follow a three-stage cycle: begin, update, end, hold
    gestures follow a two-stage cycle: begin and end. All gestures are
    identified by a unique id.

    Warning! The protocol described in this file is experimental and backward
    incompatible changes may be made. Backward compatible changes may be added
    together with the corresponding interface version bump. Backward
    incompatible changes are done by bumping the version number in the protocol
    and interface names and resetting the interface version. Once the protocol
    is to be declared stable, the 'z' prefix and the version number in the
    protocol and interface names are removed and the interface version number
    is reset.
    """

    name = "zwp_pointer_gestures_v1"
    version = 3


class ZwpPointerGesturesV1Proxy(Proxy):
    interface = ZwpPointerGesturesV1

    @ZwpPointerGesturesV1.request(
        Argument(ArgumentType.NewId, interface=ZwpPointerGestureSwipeV1),
        Argument(ArgumentType.Object, interface=WlPointer),
    )
    def get_swipe_gesture(self, pointer):
        """Get swipe gesture

        Create a swipe gesture object. See the wl_pointer_gesture_swipe
        interface for details.

        :param pointer:
        :type pointer:
            :class:`~pywayland.protocol.wayland.WlPointer`
        :returns:
            :class:`~pywayland.protocol.pointer_gestures_unstable_v1.ZwpPointerGestureSwipeV1`
        """
        id = self._marshal_constructor(0, ZwpPointerGestureSwipeV1, pointer)
        return id

    @ZwpPointerGesturesV1.request(
        Argument(ArgumentType.NewId, interface=ZwpPointerGesturePinchV1),
        Argument(ArgumentType.Object, interface=WlPointer),
    )
    def get_pinch_gesture(self, pointer):
        """Get pinch gesture

        Create a pinch gesture object. See the wl_pointer_gesture_pinch
        interface for details.

        :param pointer:
        :type pointer:
            :class:`~pywayland.protocol.wayland.WlPointer`
        :returns:
            :class:`~pywayland.protocol.pointer_gestures_unstable_v1.ZwpPointerGesturePinchV1`
        """
        id = self._marshal_constructor(1, ZwpPointerGesturePinchV1, pointer)
        return id

    @ZwpPointerGesturesV1.request(version=2)
    def release(self):
        """Destroy the pointer gesture object

        Destroy the pointer gesture object. Swipe, pinch and hold objects
        created via this gesture object remain valid.
        """
        self._marshal(2)
        self._destroy()

    @ZwpPointerGesturesV1.request(
        Argument(ArgumentType.NewId, interface=ZwpPointerGestureHoldV1),
        Argument(ArgumentType.Object, interface=WlPointer),
        version=3,
    )
    def get_hold_gesture(self, pointer):
        """Get hold gesture

        Create a hold gesture object. See the wl_pointer_gesture_hold interface
        for details.

        :param pointer:
        :type pointer:
            :class:`~pywayland.protocol.wayland.WlPointer`
        :returns:
            :class:`~pywayland.protocol.pointer_gestures_unstable_v1.ZwpPointerGestureHoldV1`
        """
        id = self._marshal_constructor(3, ZwpPointerGestureHoldV1, pointer)
        return id


class ZwpPointerGesturesV1Resource(Resource):
    interface = ZwpPointerGesturesV1


class ZwpPointerGesturesV1Global(Global):
    interface = ZwpPointerGesturesV1


ZwpPointerGesturesV1._gen_c()
ZwpPointerGesturesV1.proxy_class = ZwpPointerGesturesV1Proxy
ZwpPointerGesturesV1.resource_class = ZwpPointerGesturesV1Resource
ZwpPointerGesturesV1.global_class = ZwpPointerGesturesV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlSurface


class ZwpPointerGestureHoldV1(Interface):
    """A hold gesture object

    A hold gesture object notifies a client about a single- or multi-finger
    hold gesture detected on an indirect input device such as a touchpad. The
    gesture is usually initiated by one or more fingers being held down without
    significant movement. The precise conditions of when such a gesture is
    detected are implementation-dependent.

    In particular, this gesture may be used to cancel kinetic scrolling.

    A hold gesture consists of two stages: begin and end. Unlike pinch and
    swipe there is no update stage. There cannot be multiple simultaneous hold,
    pinch or swipe gestures on a same pointer/seat, how compositors prevent
    these situations is implementation-dependent.

    A gesture may be cancelled by the compositor or the hardware. Clients
    should not consider performing permanent or irreversible actions until the
    end of a gesture has been received.
    """

    name = "zwp_pointer_gesture_hold_v1"
    version = 3


class ZwpPointerGestureHoldV1Proxy(Proxy):
    interface = ZwpPointerGestureHoldV1

    @ZwpPointerGestureHoldV1.request(version=3)
    def destroy(self):
        """Destroy the hold gesture object
        """
        self._marshal(0)
        self._destroy()


class ZwpPointerGestureHoldV1Resource(Resource):
    interface = ZwpPointerGestureHoldV1

    @ZwpPointerGestureHoldV1.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Object, interface=WlSurface),
        Argument(ArgumentType.Uint),
        version=3,
    )
    def begin(self, serial, time, surface, fingers):
        """Multi-finger hold begin

        This event is sent when a hold gesture is detected on the device.

        :param serial:
        :type serial:
            `ArgumentType.Uint`
        :param time:
            timestamp with millisecond granularity
        :type time:
            `ArgumentType.Uint`
        :param surface:
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :param fingers:
            number of fingers
        :type fingers:
            `ArgumentType.Uint`
        """
        self._post_event(0, serial, time, surface, fingers)

    @ZwpPointerGestureHoldV1.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Int),
        version=3,
    )
    def end(self, serial, time, cancelled):
        """Multi-finger hold end

        This event is sent when a hold gesture ceases to be valid. This may
        happen when the holding fingers are lifted or the gesture is cancelled,
        for example if the fingers move past an implementation-defined
        threshold, the finger count changes or the hold gesture changes into a
        different type of gesture.

        When a gesture is cancelled, the client may need to undo state changes
        caused by this gesture. What causes a gesture to be cancelled is
        implementation-dependent.

        :param serial:
        :type serial:
            `ArgumentType.Uint`
        :param time:
            timestamp with millisecond granularity
        :type time:
            `ArgumentType.Uint`
        :param cancelled:
            1 if the gesture was cancelled, 0 otherwise
        :type cancelled:
            `ArgumentType.Int`
        """
        self._post_event(1, serial, time, cancelled)


class ZwpPointerGestureHoldV1Global(Global):
    interface = ZwpPointerGestureHoldV1


ZwpPointerGestureHoldV1._gen_c()
ZwpPointerGestureHoldV1.proxy_class = ZwpPointerGestureHoldV1Proxy
ZwpPointerGestureHoldV1.resource_class = ZwpPointerGestureHoldV1Resource
ZwpPointerGestureHoldV1.global_class = ZwpPointerGestureHoldV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright 2014 © Stephen "Lyude" Chandler Paul
# Copyright 2015-2016 © Red Hat, Inc.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from .zwp_tablet_tool_v1 import ZwpTabletToolV1
from .zwp_tablet_v1 import ZwpTabletV1


class ZwpTabletSeatV1(Interface):
    """Controller object for graphic tablet devices of a seat

    An object that provides access to the graphics tablets available on this
    seat. After binding to this interface, the compositor sends a set of
    wp_tablet_seat.tablet_added and wp_tablet_seat.tool_added events.
    """

    name = "zwp_tablet_seat_v1"
    version = 1


class ZwpTabletSeatV1Proxy(Proxy):
    interface = ZwpTabletSeatV1

    @ZwpTabletSeatV1.request()
    def destroy(self):
        """Release the memory for the tablet seat object

        Destroy the wp_tablet_seat object. Objects created from this object are
        unaffected and should be destroyed separately.
        """
        self._marshal(0)
        self._destroy()


class ZwpTabletSeatV1Resource(Resource):
    interface = ZwpTabletSeatV1

    @ZwpTabletSeatV1.event(
        Argument(ArgumentType.NewId, interface=ZwpTabletV1),
    )
    def tablet_added(self, id):
        """New device notification

        This event is sent whenever a new tablet becomes available on this
        seat. This event only provides the object id of the tablet, any static
        information about the tablet (device name, vid/pid, etc.) is sent
        through the wp_tablet interface.

        :param id:
            the newly added graphics tablet
        :type id:
            :class:`~pywayland.protocol.tablet_unstable_v1.ZwpTabletV1`
        """
        self._post_event(0, id)

    @ZwpTabletSeatV1.event(
        Argument(ArgumentType.NewId, interface=ZwpTabletToolV1),
    )
    def tool_added(self, id):
        """A new tool has been used with a tablet

        This event is sent whenever a tool that has not previously been used
        with a tablet comes into use. This event only provides the object id of
        the tool; any static information about the tool (capabilities, type,
        etc.) is sent through the wp_tablet_tool interface.

        :param id:
            the newly added tablet tool
        :type id:
            :class:`~pywayland.protocol.tablet_unstable_v1.ZwpTabletToolV1`
        """
        self._post_event(1, id)


class ZwpTabletSeatV1Global(Global):
    interface = ZwpTabletSeatV1


ZwpTabletSeatV1._gen_c()
ZwpTabletSeatV1.proxy_class = ZwpTabletSeatV1Proxy
ZwpTabletSeatV1.resource_class = ZwpTabletSeatV1Resource
ZwpTabletSeatV1.global_class = ZwpTabletSeatV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2014      Jonas Ådahl
# Copyright © 2015      Red Hat Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlPointer
from .zwp_relative_pointer_v1 import ZwpRelativePointerV1


class ZwpRelativePointerManagerV1(Interface):
    """Get relative pointer objects

    A global interface used for getting the relative pointer object for a given
    pointer.
    """

    name = "zwp_relative_pointer_manager_v1"
    version = 1


class ZwpRelativePointerManagerV1Proxy(Proxy):
    interface = ZwpRelativePointerManagerV1

    @ZwpRelativePointerManagerV1.request()
    def destroy(self):
        """Destroy the relative pointer manager object

        Used by the client to notify the server that it will no longer use this
        relative pointer manager object.
        """
        self._marshal(0)
        self._destroy()

    @ZwpRelativePointerManagerV1.request(
        Argument(ArgumentType.NewId, interface=ZwpRelativePointerV1),
        Argument(ArgumentType.Object, interface=WlPointer),
    )
    def get_relative_pointer(self, pointer):
        """Get a relative pointer object

        Create a relative pointer interface given a
        :class:`~pywayland.protocol.wayland.WlPointer` object. See the
        wp_relative_pointer interface for more details.

        :param pointer:
        :type pointer:
            :class:`~pywayland.protocol.wayland.WlPointer`
        :returns:
            :class:`~pywayland.protocol.relative_pointer_unstable_v1.ZwpRelativePointerV1`
        """
        id = self._marshal_constructor(1, ZwpRelativePointerV1, pointer)
        return id


class ZwpRelativePointerManagerV1Resource(Resource):
    interface = ZwpRelativePointerManagerV1


class ZwpRelativePointerManagerV1Global(Global):
    interface = ZwpRelativePointerManagerV1


ZwpRelativePointerManagerV1._gen_c()
ZwpRelativePointerManagerV1.proxy_class = ZwpRelativePointerManagerV1Proxy
ZwpRelativePointerManagerV1.resource_class = ZwpRelativePointerManagerV1Resource
ZwpRelativePointerManagerV1.global_class = ZwpRelativePointerManagerV1Global



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Optional, Union, TYPE_CHECKING
from weakref import WeakSet

from pywayland import ffi, lib
from pywayland.client.eventqueue import EventQueue
from pywayland.utils import ensure_valid
from pywayland.protocol.wayland import WlDisplay

if TYPE_CHECKING:
    # introduced in standard library in Python 3.8
    from typing_extensions import Literal  # noqa: F401


class Display(WlDisplay.proxy_class):  # type: ignore
    """Represents a connection to the compositor

    A :class:`Display` object represents a client connection to a Wayland
    compositor.  The connection and the corresponding Wayland object are
    created with :func:`Display.connect`.  The display must be connected before
    it can be used.  A connection is terminated using
    :func:`Display.disconnect`.

    A :class:`Display` is also used as the
    :class:`~pywayland.client.proxy.Proxy` for the
    :class:`pywayland.protocol.wayland.WlDisplay` protocol object on the
    compositor side.

    A :class:`Display` object handles all the data sent from and to the
    compositor.  When a :class:`~pywayland.client.proxy.Proxy` marshals a
    request, it will write its wire representation to the display's write
    buffer. The data is sent to the compositor when the client calls
    :func:`flush`.

    Incoming data is handled in two steps: queueing and dispatching. In the
    queue step, the data coming from the display fd is interpreted and added to
    a queue. On the dispatch step, the handler for the incoming event set by
    the client on the corresponding :class:`~pywayland.client.proxy.Proxy` is
    called.

    A :class:`Display` has at least one event queue, called the `default
    queue`.  Clients can create additional event queues with
    :func:`Display.create_queue` and assign
    :class:`~pywayland.client.proxy.Proxy`'s to it. Events occurring in a
    particular proxy are always queued in its assigned queue.  A client can
    ensure that a certain assumption, such as holding a lock or running from a
    given thread, is true when a proxy event handler is called by assigning
    that proxy to an event queue and making sure that this queue is only
    dispatched when the assumption holds.

    The default queue is dispatched by calling :func:`Display.dispatch`.  This
    will dispatch any events queued on the default queue and attempt to read
    from the display fd if it's empty. Events read are then queued on the
    appropriate queues according to the proxy assignment.

    A user created queue is dispatched with :func:`Display.dispatch_queue`.
    This function behaves exactly the same as :func:`Display.dispatch` but it
    dispatches given queue instead of the default queue.

    A real world example of event queue usage is Mesa's implementation of
    glSwapBuffers() for the Wayland platform. This function might need to block
    until a frame callback is received, but dispatching the default ueue could
    cause an event handler on the client to start drawing gain.  This problem
    is solved using another event queue, so that only the events handled by the
    EGL code are dispatched during the block.

    :param name_or_fd:
        Either the name of the display to create or the file descriptor to
        connect the display to.  If not specified, then use the default name,
        generally ``wayland-0``
    :type name_or_fd:
        ``int`` or ``str``
    """

    def __init__(self, name_or_fd: Optional[Union[int, str]] = None) -> None:
        """Constructor for the Display object"""
        super().__init__(None)

        self._children: WeakSet = WeakSet()
        self._name_or_fd = name_or_fd
        self._ptr: Optional["ffi.DisplayCData"] = None

    def __enter__(self) -> "Display":
        """Connect to the display in a context manager"""
        self.connect()
        return self

    def __exit__(self, exc_type, exc_value, traceback) -> "Literal[False]":
        """Disconnect from the display"""
        self.disconnect()
        return False

    def connect(self) -> None:
        """Connect to a Wayland display

        Connect to the Wayland display by name of fd.  An `int` parameter opens
        the connection using the file descriptor.  The :class:`Display` takes
        ownership of the fd and will close it when the display is destroyed.
        The fd will also be closed in case of failure.  A string will open the
        display of the given name.  If name is ``None``, its value will be
        replaced with the ``WAYLAND_DISPLAY`` environment variable if it is
        set, otherwise display ``"wayland-0"`` will be used.
        """
        if isinstance(self._name_or_fd, int):
            # argument passed is a file descriptor
            self._ptr = lib.wl_display_connect_to_fd(self._name_or_fd)
        else:
            # connect using string by name, or use default
            if self._name_or_fd is None:
                name: Union["ffi.CData", bytes] = ffi.NULL
            else:
                name = self._name_or_fd.encode()
            self._ptr = lib.wl_display_connect(name)

        if self._ptr == ffi.NULL:
            raise ValueError("Unable to connect to display")

        self._ptr = ffi.gc(self._ptr, lib.wl_display_disconnect)

    def disconnect(self) -> None:
        """Close a connection to a Wayland display

        Close the connection to display and free all resources associated with
        it.
        """
        if self._ptr:
            # we need to be sure the event queues and proxies are destroyed
            # before we disconnect the client
            for obj in self._children:
                if not obj.destroyed:
                    obj.destroy()

            # run destructor and remove it
            ffi.release(self._ptr)
            self._ptr = None

    _destroy = disconnect

    @ensure_valid
    def get_fd(self) -> int:
        """Get a display context's file descriptor

        Return the file descriptor associated with a display so it can be
        integrated into the client's main loop.
        """
        assert self._ptr is not None
        return lib.wl_display_get_fd(self._ptr)

    @ensure_valid
    def dispatch(
        self, *, block: bool = False, queue: Optional[EventQueue] = None
    ) -> int:
        """Process incoming events

        If block is `False`, it does not attempt to read the display fd or
        event queue and simply returns zero if the queue is empty.

        If the given queue is empty and `block` is `True`, this function blocks
        until there are events to be read from the display fd. Events are read
        and queued on the appropriate event queues. Finally, events on the
        default event queue are dispatched.

        .. note::

            It is not possible to check if there are events on the queue or
            not.
        """
        assert self._ptr is not None
        if block:
            if queue is None:
                ret = lib.wl_display_dispatch(self._ptr)
            else:
                assert queue._ptr is not None
                ret = lib.wl_display_dispatch_queue(self._ptr, queue._ptr)
        else:
            if queue is None:
                ret = lib.wl_display_dispatch_pending(self._ptr)
            else:
                assert queue._ptr is not None
                ret = lib.wl_display_dispatch_queue_pending(self._ptr, queue._ptr)

        if ret == -1:
            err = lib.wl_display_get_error(self._ptr)
            raise RuntimeError("Failed with error: {}".format(err))

        return ret

    @ensure_valid
    def roundtrip(self, *, queue: Optional[EventQueue] = None) -> int:
        """Block until all pending request are processed by the server

        This function blocks until the server has processed all currently
        issued requests by sending a request to the display server and waiting
        for a reply before returning.

        This function uses wl_display_dispatch_queue() internally. It is not
        allowed to call this function while the thread is being prepared for
        reading events, and doing so will cause a dead lock.

        .. note::

            This function may dispatch other events being received on the
            default queue.

        :param queue: The queue on which to run the roundtrip, if not given,
                      uses the default queue.
        :type queue: :class:`~pywayland.client.EventQueue`
        :returns: The number of dispatched events on success or -1 on failure
        """
        assert self._ptr is not None
        if queue is None:
            return lib.wl_display_roundtrip(self._ptr)
        else:
            assert queue._ptr is not None
            return lib.wl_display_roundtrip_queue(self._ptr, queue._ptr)

    @ensure_valid
    def read(self, *, queue: Optional[EventQueue] = None) -> None:
        """Read events from display file descriptor

        Calling this function will result in data available on the display file
        descriptor being read and read events will be queued on their
        corresponding event queues.

        :param queue: If specified, queue the events onto the given event
                      queue, otherwise the default display queue will be used.
        """
        assert self._ptr is not None
        while True:
            if queue is None:
                prepared = lib.wl_display_prepare_read(self._ptr)
            else:
                assert queue._ptr is not None
                prepared = lib.wl_display_prepare_read_queue(self._ptr, queue._ptr)

            if prepared == 0:
                break

            # TODO: add a blocking/non-blocking condition here

            self.dispatch(block=False, queue=queue)

        status = lib.wl_display_read_events(self._ptr)
        if status != 0:
            raise RuntimeError("Failed to read events")

    @ensure_valid
    def flush(self) -> int:
        """Send all buffered requests on the display to the server

        Send all buffered data on the client side to the server. Clients
        should call this function before blocking. On success, the number
        of bytes sent to the server is returned. On failure, this
        function returns -1 and errno is set appropriately.

        :func:`Display.flush` never blocks.  It will write as much data as
        possible, but if all data could not be written, errno will be set to
        EAGAIN and -1 returned.  In that case, use poll on the display file
        descriptor to wait for it to become writable again.
        """
        assert self._ptr is not None
        return lib.wl_display_flush(self._ptr)



# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from .wl_buffer import WlBuffer  # noqa: F401
from .wl_callback import WlCallback  # noqa: F401
from .wl_compositor import WlCompositor  # noqa: F401
from .wl_data_device import WlDataDevice  # noqa: F401
from .wl_data_device_manager import WlDataDeviceManager  # noqa: F401
from .wl_data_offer import WlDataOffer  # noqa: F401
from .wl_data_source import WlDataSource  # noqa: F401
from .wl_display import WlDisplay  # noqa: F401
from .wl_keyboard import WlKeyboard  # noqa: F401
from .wl_output import WlOutput  # noqa: F401
from .wl_pointer import WlPointer  # noqa: F401
from .wl_region import WlRegion  # noqa: F401
from .wl_registry import WlRegistry  # noqa: F401
from .wl_seat import WlSeat  # noqa: F401
from .wl_shell import WlShell  # noqa: F401
from .wl_shell_surface import WlShellSurface  # noqa: F401
from .wl_shm import WlShm  # noqa: F401
from .wl_shm_pool import WlShmPool  # noqa: F401
from .wl_subcompositor import WlSubcompositor  # noqa: F401
from .wl_subsurface import WlSubsurface  # noqa: F401
from .wl_surface import WlSurface  # noqa: F401
from .wl_touch import WlTouch  # noqa: F401



# This file has been autogenerated by the pywayland scanner

# Copyright © 2015-2016 Red Hat Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from .zxdg_imported_v2 import ZxdgImportedV2


class ZxdgImporterV2(Interface):
    """Interface for importing surfaces

    A global interface used for importing surfaces exported by xdg_exporter.
    With this interface, a client can create a reference to a surface of
    another client.
    """

    name = "zxdg_importer_v2"
    version = 1


class ZxdgImporterV2Proxy(Proxy):
    interface = ZxdgImporterV2

    @ZxdgImporterV2.request()
    def destroy(self):
        """Destroy the xdg_importer object

        Notify the compositor that the xdg_importer object will no longer be
        used.
        """
        self._marshal(0)
        self._destroy()

    @ZxdgImporterV2.request(
        Argument(ArgumentType.NewId, interface=ZxdgImportedV2),
        Argument(ArgumentType.String),
    )
    def import_toplevel(self, handle):
        """Import a toplevel surface

        The import_toplevel request imports a surface from any client given a
        handle retrieved by exporting said surface using
        xdg_exporter.export_toplevel. When called, a new xdg_imported object
        will be created. This new object represents the imported surface, and
        the importing client can manipulate its relationship using it. See
        xdg_imported for details.

        :param handle:
            the exported surface handle
        :type handle:
            `ArgumentType.String`
        :returns:
            :class:`~pywayland.protocol.xdg_foreign_unstable_v2.ZxdgImportedV2`
            -- the new xdg_imported object
        """
        id = self._marshal_constructor(1, ZxdgImportedV2, handle)
        return id


class ZxdgImporterV2Resource(Resource):
    interface = ZxdgImporterV2


class ZxdgImporterV2Global(Global):
    interface = ZxdgImporterV2


ZxdgImporterV2._gen_c()
ZxdgImporterV2.proxy_class = ZxdgImporterV2Proxy
ZxdgImporterV2.resource_class = ZxdgImporterV2Resource
ZxdgImporterV2.global_class = ZxdgImporterV2Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2017 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlKeyboard
from ..wayland import WlPointer
from ..wayland import WlTouch
from .zwp_input_timestamps_v1 import ZwpInputTimestampsV1


class ZwpInputTimestampsManagerV1(Interface):
    """Context object for high-resolution input timestamps

    A global interface used for requesting high-resolution timestamps for input
    events.
    """

    name = "zwp_input_timestamps_manager_v1"
    version = 1


class ZwpInputTimestampsManagerV1Proxy(Proxy):
    interface = ZwpInputTimestampsManagerV1

    @ZwpInputTimestampsManagerV1.request()
    def destroy(self):
        """Destroy the input timestamps manager object

        Informs the server that the client will no longer be using this
        protocol object. Existing objects created by this object are not
        affected.
        """
        self._marshal(0)
        self._destroy()

    @ZwpInputTimestampsManagerV1.request(
        Argument(ArgumentType.NewId, interface=ZwpInputTimestampsV1),
        Argument(ArgumentType.Object, interface=WlKeyboard),
    )
    def get_keyboard_timestamps(self, keyboard):
        """Subscribe to high-resolution keyboard timestamp events

        Creates a new input timestamps object that represents a subscription to
        high-resolution timestamp events for all
        :class:`~pywayland.protocol.wayland.WlKeyboard` events that carry a
        timestamp.

        If the associated :class:`~pywayland.protocol.wayland.WlKeyboard`
        object is invalidated, either through client action (e.g. release) or
        server-side changes, the input timestamps object becomes inert and the
        client should destroy it by calling
        :func:`ZwpInputTimestampsV1.destroy()
        <pywayland.protocol.input_timestamps_unstable_v1.ZwpInputTimestampsV1.destroy>`.

        :param keyboard:
            the :class:`~pywayland.protocol.wayland.WlKeyboard` object for
            which to get timestamp events
        :type keyboard:
            :class:`~pywayland.protocol.wayland.WlKeyboard`
        :returns:
            :class:`~pywayland.protocol.input_timestamps_unstable_v1.ZwpInputTimestampsV1`
        """
        id = self._marshal_constructor(1, ZwpInputTimestampsV1, keyboard)
        return id

    @ZwpInputTimestampsManagerV1.request(
        Argument(ArgumentType.NewId, interface=ZwpInputTimestampsV1),
        Argument(ArgumentType.Object, interface=WlPointer),
    )
    def get_pointer_timestamps(self, pointer):
        """Subscribe to high-resolution pointer timestamp events

        Creates a new input timestamps object that represents a subscription to
        high-resolution timestamp events for all
        :class:`~pywayland.protocol.wayland.WlPointer` events that carry a
        timestamp.

        If the associated :class:`~pywayland.protocol.wayland.WlPointer` object
        is invalidated, either through client action (e.g. release) or server-
        side changes, the input timestamps object becomes inert and the client
        should destroy it by calling :func:`ZwpInputTimestampsV1.destroy()
        <pywayland.protocol.input_timestamps_unstable_v1.ZwpInputTimestampsV1.destroy>`.

        :param pointer:
            the :class:`~pywayland.protocol.wayland.WlPointer` object for which
            to get timestamp events
        :type pointer:
            :class:`~pywayland.protocol.wayland.WlPointer`
        :returns:
            :class:`~pywayland.protocol.input_timestamps_unstable_v1.ZwpInputTimestampsV1`
        """
        id = self._marshal_constructor(2, ZwpInputTimestampsV1, pointer)
        return id

    @ZwpInputTimestampsManagerV1.request(
        Argument(ArgumentType.NewId, interface=ZwpInputTimestampsV1),
        Argument(ArgumentType.Object, interface=WlTouch),
    )
    def get_touch_timestamps(self, touch):
        """Subscribe to high-resolution touch timestamp events

        Creates a new input timestamps object that represents a subscription to
        high-resolution timestamp events for all
        :class:`~pywayland.protocol.wayland.WlTouch` events that carry a
        timestamp.

        If the associated :class:`~pywayland.protocol.wayland.WlTouch` object
        becomes invalid, either through client action (e.g. release) or server-
        side changes, the input timestamps object becomes inert and the client
        should destroy it by calling :func:`ZwpInputTimestampsV1.destroy()
        <pywayland.protocol.input_timestamps_unstable_v1.ZwpInputTimestampsV1.destroy>`.

        :param touch:
            the :class:`~pywayland.protocol.wayland.WlTouch` object for which
            to get timestamp events
        :type touch:
            :class:`~pywayland.protocol.wayland.WlTouch`
        :returns:
            :class:`~pywayland.protocol.input_timestamps_unstable_v1.ZwpInputTimestampsV1`
        """
        id = self._marshal_constructor(3, ZwpInputTimestampsV1, touch)
        return id


class ZwpInputTimestampsManagerV1Resource(Resource):
    interface = ZwpInputTimestampsManagerV1


class ZwpInputTimestampsManagerV1Global(Global):
    interface = ZwpInputTimestampsManagerV1


ZwpInputTimestampsManagerV1._gen_c()
ZwpInputTimestampsManagerV1.proxy_class = ZwpInputTimestampsManagerV1Proxy
ZwpInputTimestampsManagerV1.resource_class = ZwpInputTimestampsManagerV1Resource
ZwpInputTimestampsManagerV1.global_class = ZwpInputTimestampsManagerV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2015, 2016 Red Hat
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource


class ZwpPrimarySelectionSourceV1(Interface):
    """Offer to replace the contents of the primary selection

    The source side of a wp_primary_selection_offer, it provides a way to
    describe the offered data and respond to requests to transfer the requested
    contents of the primary selection clipboard.
    """

    name = "zwp_primary_selection_source_v1"
    version = 1


class ZwpPrimarySelectionSourceV1Proxy(Proxy):
    interface = ZwpPrimarySelectionSourceV1

    @ZwpPrimarySelectionSourceV1.request(
        Argument(ArgumentType.String),
    )
    def offer(self, mime_type):
        """Add an offered mime type

        This request adds a mime type to the set of mime types advertised to
        targets. Can be called several times to offer multiple types.

        :param mime_type:
        :type mime_type:
            `ArgumentType.String`
        """
        self._marshal(0, mime_type)

    @ZwpPrimarySelectionSourceV1.request()
    def destroy(self):
        """Destroy the primary selection source

        Destroy the primary selection source.
        """
        self._marshal(1)
        self._destroy()


class ZwpPrimarySelectionSourceV1Resource(Resource):
    interface = ZwpPrimarySelectionSourceV1

    @ZwpPrimarySelectionSourceV1.event(
        Argument(ArgumentType.String),
        Argument(ArgumentType.FileDescriptor),
    )
    def send(self, mime_type, fd):
        """Send the primary selection contents

        Request for the current primary selection contents from the client.
        Send the specified mime type over the passed file descriptor, then
        close it.

        :param mime_type:
        :type mime_type:
            `ArgumentType.String`
        :param fd:
        :type fd:
            `ArgumentType.FileDescriptor`
        """
        self._post_event(0, mime_type, fd)

    @ZwpPrimarySelectionSourceV1.event()
    def cancelled(self):
        """Request for primary selection contents was canceled

        This primary selection source is no longer valid. The client should
        clean up and destroy this primary selection source.
        """
        self._post_event(1)


class ZwpPrimarySelectionSourceV1Global(Global):
    interface = ZwpPrimarySelectionSourceV1


ZwpPrimarySelectionSourceV1._gen_c()
ZwpPrimarySelectionSourceV1.proxy_class = ZwpPrimarySelectionSourceV1Proxy
ZwpPrimarySelectionSourceV1.resource_class = ZwpPrimarySelectionSourceV1Resource
ZwpPrimarySelectionSourceV1.global_class = ZwpPrimarySelectionSourceV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource


class WlRegistry(Interface):
    """Global registry object

    The singleton global registry object.  The server has a number of global
    objects that are available to all clients.  These objects typically
    represent an actual object in the server (for example, an input device) or
    they are singleton objects that provide extension functionality.

    When a client creates a registry object, the registry object will emit a
    global event for each global currently in the registry.  Globals come and
    go as a result of device or monitor hotplugs, reconfiguration or other
    events, and the registry will send out global and global_remove events to
    keep the client up to date with the changes.  To mark the end of the
    initial burst of events, the client can use the :func:`WlDisplay.sync()
    <pywayland.protocol.wayland.WlDisplay.sync>` request immediately after
    calling :func:`WlDisplay.get_registry()
    <pywayland.protocol.wayland.WlDisplay.get_registry>`.

    A client can bind to a global object by using the bind request.  This
    creates a client-side handle that lets the object emit events to the client
    and lets the client invoke requests on the object.
    """

    name = "wl_registry"
    version = 1


class WlRegistryProxy(Proxy):
    interface = WlRegistry

    @WlRegistry.request(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.NewId),
    )
    def bind(self, name, interface, version):
        """Bind an object to the display

        Binds a new, client-created object to the server using the specified
        name as the identifier.

        :param name:
            unique numeric name of the object
        :type name:
            `ArgumentType.Uint`
        :param interface:
            Interface name
        :type interface:
            `string`
        :param version:
            Interface version
        :type version:
            `int`
        :returns:
            :class:`pywayland.client.proxy.Proxy` of specified Interface --
            bounded object
        """
        id = self._marshal_constructor(0, interface, name, interface.name, version)
        return id


class WlRegistryResource(Resource):
    interface = WlRegistry

    @WlRegistry.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.String),
        Argument(ArgumentType.Uint),
    )
    def global_(self, name, interface, version):
        """Announce global object

        Notify the client of global objects.

        The event notifies the client that a global object with the given name
        is now available, and it implements the given version of the given
        interface.

        :param name:
            numeric name of the global object
        :type name:
            `ArgumentType.Uint`
        :param interface:
            interface implemented by the object
        :type interface:
            `ArgumentType.String`
        :param version:
            interface version
        :type version:
            `ArgumentType.Uint`
        """
        self._post_event(0, name, interface, version)

    @WlRegistry.event(
        Argument(ArgumentType.Uint),
    )
    def global_remove(self, name):
        """Announce removal of global object

        Notify the client of removed global objects.

        This event notifies the client that the global identified by name is no
        longer available.  If the client bound to the global using the bind
        request, the client should now destroy that object.

        The object remains valid and requests to the object will be ignored
        until the client destroys it, to avoid races between the global going
        away and a client sending a request to it.

        :param name:
            numeric name of the global object
        :type name:
            `ArgumentType.Uint`
        """
        self._post_event(1, name)


class WlRegistryGlobal(Global):
    interface = WlRegistry


WlRegistry._gen_c()
WlRegistry.proxy_class = WlRegistryProxy
WlRegistry.resource_class = WlRegistryResource
WlRegistry.global_class = WlRegistryGlobal



# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource


class WlDataOffer(Interface):
    """Offer to transfer data

    A :class:`WlDataOffer` represents a piece of data offered for transfer by
    another client (the source client).  It is used by the copy-and-paste and
    drag-and-drop mechanisms.  The offer describes the different mime types
    that the data can be converted to and provides the mechanism for
    transferring the data directly from the source client.
    """

    name = "wl_data_offer"
    version = 3

    class error(enum.IntEnum):
        invalid_finish = 0
        invalid_action_mask = 1
        invalid_action = 2
        invalid_offer = 3


class WlDataOfferProxy(Proxy):
    interface = WlDataOffer

    @WlDataOffer.request(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.String, nullable=True),
    )
    def accept(self, serial, mime_type):
        """Accept one of the offered mime types

        Indicate that the client can accept the given mime type, or NULL for
        not accepted.

        For objects of version 2 or older, this request is used by the client
        to give feedback whether the client can receive the given mime type, or
        NULL if none is accepted; the feedback does not determine whether the
        drag-and-drop operation succeeds or not.

        For objects of version 3 or newer, this request determines the final
        result of the drag-and-drop operation. If the end result is that no
        mime types were accepted, the drag-and-drop operation will be cancelled
        and the corresponding drag source will receive
        :func:`WlDataSource.cancelled()
        <pywayland.protocol.wayland.WlDataSource.cancelled>`. Clients may still
        use this event in conjunction with :func:`WlDataSource.action()
        <pywayland.protocol.wayland.WlDataSource.action>` for feedback.

        :param serial:
            serial number of the accept request
        :type serial:
            `ArgumentType.Uint`
        :param mime_type:
            mime type accepted by the client
        :type mime_type:
            `ArgumentType.String` or `None`
        """
        self._marshal(0, serial, mime_type)

    @WlDataOffer.request(
        Argument(ArgumentType.String),
        Argument(ArgumentType.FileDescriptor),
    )
    def receive(self, mime_type, fd):
        """Request that the data is transferred

        To transfer the offered data, the client issues this request and
        indicates the mime type it wants to receive.  The transfer happens
        through the passed file descriptor (typically created with the pipe
        system call).  The source client writes the data in the mime type
        representation requested and then closes the file descriptor.

        The receiving client reads from the read end of the pipe until EOF and
        then closes its end, at which point the transfer is complete.

        This request may happen multiple times for different mime types, both
        before and after :func:`WlDataDevice.drop()
        <pywayland.protocol.wayland.WlDataDevice.drop>`. Drag-and-drop
        destination clients may preemptively fetch data or examine it more
        closely to determine acceptance.

        :param mime_type:
            mime type desired by receiver
        :type mime_type:
            `ArgumentType.String`
        :param fd:
            file descriptor for data transfer
        :type fd:
            `ArgumentType.FileDescriptor`
        """
        self._marshal(1, mime_type, fd)

    @WlDataOffer.request()
    def destroy(self):
        """Destroy data offer

        Destroy the data offer.
        """
        self._marshal(2)
        self._destroy()

    @WlDataOffer.request(version=3)
    def finish(self):
        """The offer will no longer be used

        Notifies the compositor that the drag destination successfully finished
        the drag-and-drop operation.

        Upon receiving this request, the compositor will emit
        :func:`WlDataSource.dnd_finished()
        <pywayland.protocol.wayland.WlDataSource.dnd_finished>` on the drag
        source client.

        It is a client error to perform other requests than
        :func:`WlDataOffer.destroy()` after this one. It is also an error to
        perform this request after a NULL mime type has been set in
        :func:`WlDataOffer.accept()` or no action was received through
        :func:`WlDataOffer.action()`.

        If :func:`WlDataOffer.finish()` request is received for a non drag and
        drop operation, the invalid_finish protocol error is raised.
        """
        self._marshal(3)

    @WlDataOffer.request(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        version=3,
    )
    def set_actions(self, dnd_actions, preferred_action):
        """Set the available/preferred drag-and-drop actions

        Sets the actions that the destination side client supports for this
        operation. This request may trigger the emission of
        :func:`WlDataSource.action()
        <pywayland.protocol.wayland.WlDataSource.action>` and
        :func:`WlDataOffer.action()` events if the compositor needs to change
        the selected action.

        This request can be called multiple times throughout the drag-and-drop
        operation, typically in response to :func:`WlDataDevice.enter()
        <pywayland.protocol.wayland.WlDataDevice.enter>` or
        :func:`WlDataDevice.motion()
        <pywayland.protocol.wayland.WlDataDevice.motion>` events.

        This request determines the final result of the drag-and-drop
        operation. If the end result is that no action is accepted, the drag
        source will receive :func:`WlDataSource.cancelled()
        <pywayland.protocol.wayland.WlDataSource.cancelled>`.

        The dnd_actions argument must contain only values expressed in the
        :func:`WlDataDeviceManager.dnd_actions()
        <pywayland.protocol.wayland.WlDataDeviceManager.dnd_actions>` enum, and
        the preferred_action argument must only contain one of those values
        set, otherwise it will result in a protocol error.

        While managing an "ask" action, the destination drag-and-drop client
        may perform further :func:`WlDataOffer.receive()` requests, and is
        expected to perform one last :func:`WlDataOffer.set_actions()` request
        with a preferred action other than "ask" (and optionally
        :func:`WlDataOffer.accept()`) before requesting
        :func:`WlDataOffer.finish()`, in order to convey the action selected by
        the user. If the preferred action is not in the
        :func:`WlDataOffer.source_actions()` mask, an error will be raised.

        If the "ask" action is dismissed (e.g. user cancellation), the client
        is expected to perform :func:`WlDataOffer.destroy()` right away.

        This request can only be made on drag-and-drop offers, a protocol error
        will be raised otherwise.

        :param dnd_actions:
            actions supported by the destination client
        :type dnd_actions:
            `ArgumentType.Uint`
        :param preferred_action:
            action preferred by the destination client
        :type preferred_action:
            `ArgumentType.Uint`
        """
        self._marshal(4, dnd_actions, preferred_action)


class WlDataOfferResource(Resource):
    interface = WlDataOffer

    @WlDataOffer.event(
        Argument(ArgumentType.String),
    )
    def offer(self, mime_type):
        """Advertise offered mime type

        Sent immediately after creating the :class:`WlDataOffer` object.  One
        event per offered mime type.

        :param mime_type:
            offered mime type
        :type mime_type:
            `ArgumentType.String`
        """
        self._post_event(0, mime_type)

    @WlDataOffer.event(
        Argument(ArgumentType.Uint),
        version=3,
    )
    def source_actions(self, source_actions):
        """Notify the source-side available actions

        This event indicates the actions offered by the data source. It will be
        sent right after :func:`WlDataDevice.enter()
        <pywayland.protocol.wayland.WlDataDevice.enter>`, or anytime the source
        side changes its offered actions through
        :func:`WlDataSource.set_actions()
        <pywayland.protocol.wayland.WlDataSource.set_actions>`.

        :param source_actions:
            actions offered by the data source
        :type source_actions:
            `ArgumentType.Uint`
        """
        self._post_event(1, source_actions)

    @WlDataOffer.event(
        Argument(ArgumentType.Uint),
        version=3,
    )
    def action(self, dnd_action):
        """Notify the selected action

        This event indicates the action selected by the compositor after
        matching the source/destination side actions. Only one action (or none)
        will be offered here.

        This event can be emitted multiple times during the drag-and-drop
        operation in response to destination side action changes through
        :func:`WlDataOffer.set_actions()`.

        This event will no longer be emitted after :func:`WlDataDevice.drop()
        <pywayland.protocol.wayland.WlDataDevice.drop>` happened on the drag-
        and-drop destination, the client must honor the last action received,
        or the last preferred one set through :func:`WlDataOffer.set_actions()`
        when handling an "ask" action.

        Compositors may also change the selected action on the fly, mainly in
        response to keyboard modifier changes during the drag-and-drop
        operation.

        The most recent action received is always the valid one. Prior to
        receiving :func:`WlDataDevice.drop()
        <pywayland.protocol.wayland.WlDataDevice.drop>`, the chosen action may
        change (e.g. due to keyboard modifiers being pressed). At the time of
        receiving :func:`WlDataDevice.drop()
        <pywayland.protocol.wayland.WlDataDevice.drop>` the drag-and-drop
        destination must honor the last action received.

        Action changes may still happen after :func:`WlDataDevice.drop()
        <pywayland.protocol.wayland.WlDataDevice.drop>`, especially on "ask"
        actions, where the drag-and-drop destination may choose another action
        afterwards. Action changes happening at this stage are always the
        result of inter-client negotiation, the compositor shall no longer be
        able to induce a different action.

        Upon "ask" actions, it is expected that the drag-and-drop destination
        may potentially choose a different action and/or mime type, based on
        :func:`WlDataOffer.source_actions()` and finally chosen by the user
        (e.g. popping up a menu with the available options). The final
        :func:`WlDataOffer.set_actions()` and :func:`WlDataOffer.accept()`
        requests must happen before the call to :func:`WlDataOffer.finish()`.

        :param dnd_action:
            action selected by the compositor
        :type dnd_action:
            `ArgumentType.Uint`
        """
        self._post_event(2, dnd_action)


class WlDataOfferGlobal(Global):
    interface = WlDataOffer


WlDataOffer._gen_c()
WlDataOffer.proxy_class = WlDataOfferProxy
WlDataOffer.resource_class = WlDataOfferResource
WlDataOffer.global_class = WlDataOfferGlobal



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import subprocess
import sys

from setuptools import setup
from setuptools.command.install import install
from setuptools.command.sdist import sdist

try:
    from wheel.bdist_wheel import bdist_wheel
except ImportError:
    bdist_wheel = None

# we need this to import the version and scanner module directly so we can
# build the protocol before the cffi module has been compiled
sys.path.insert(0, "pywayland")

default_xml_file = "/usr/share/wayland/wayland.xml"


def get_protocol_command(klass):
    class ProtocolCommand(klass):
        user_options = [
            ("xml-file=", None, "Location of wayland.xml protocol file"),
            ("output-dir=", None, "Output location for protocol python files"),
            (
                "wayland-protocols",
                None,
                "Force generation of external protocols from wayland-protocols",
            ),
            (
                "no-wayland-protocols",
                None,
                "Disable generation of external protocols from wayland-protocols",
            ),
        ] + klass.user_options
        boolean_options = [
            "wayland-protocols",
            "no-wayland-protocols",
        ] + klass.boolean_options

        def initialize_options(self):
            from scanner.__main__ import pkgconfig

            # try to figure out where the main wayland protocols are installed
            try:
                data_dir = pkgconfig("wayland-scanner", "pkgdatadir")
            except subprocess.CalledProcessError:
                # silently fallback to the default
                self.xml_file = default_xml_file
            else:
                self.xml_file = os.path.join(data_dir, "wayland.xml")

            self.output_dir = "./pywayland/protocol"
            self.wayland_protocols = False
            self.no_wayland_protocols = False

            klass.initialize_options(self)

        def finalize_options(self):
            assert os.path.exists(self.xml_file), (
                "Specified Wayland protocol file, {}, does not exist "
                "please specify valid protocol file".format(self.xml_file)
            )

            klass.finalize_options(self)

        def run(self):
            from scanner import Protocol
            from scanner.__main__ import get_wayland_protocols

            # Generate the wayland interface by default
            input_files = [self.xml_file]

            # Unless users says don't build protocols, try to build them
            if not self.no_wayland_protocols:
                try:
                    protocol_files = get_wayland_protocols()
                    input_files += protocol_files
                except Exception:
                    # but only complain if we ask specifically to build them
                    if self.wayland_protocols:
                        raise

            # Ensure the output dir exists
            if not os.path.isdir(self.output_dir):
                os.makedirs(self.output_dir, 0o775)

            # Run and scan all the above found xml files
            protocols = [Protocol.parse_file(input_xml) for input_xml in input_files]
            protocol_imports = {
                interface.name: protocol.name
                for protocol in protocols
                for interface in protocol.interface
            }

            for protocol in protocols:
                protocol.output(self.output_dir, protocol_imports)

            self.distribution.packages.extend(
                f"pywayland.protocol.{protocol.name}" for protocol in protocols
            )

            klass.run(self)

    return ProtocolCommand


InstallCommand = get_protocol_command(install)
SdistCommand = get_protocol_command(sdist)

cmdclass = {
    "install": InstallCommand,
    "sdist": SdistCommand,
}

if bdist_wheel is not None:
    BdistWheelCommand = get_protocol_command(bdist_wheel)
    cmdclass["bdist_wheel"] = BdistWheelCommand

# For the purposes of uploading to PyPI, we'll get the version of Wayland here
with open("README.rst") as f:
    rst_input = f.read().strip().split("\n")
try:
    from pywayland import (
        __wayland_version__ as wayland_version,
        __version__ as pywayland_version,
    )

    version_tag = f"v{pywayland_version}"
except Exception:
    pass
else:
    version = f"Built against Wayland {wayland_version}\n"
    rst_input.insert(3, version)

    # replace all of the badges and links to point to the current version
    rst_input = rst_input[:-9]
    rst_input.extend(
        [
            f".. |ci| image:: https://github.com/flacjacket/pywayland/actions/workflows/ci.yml/badge.svg?branch={version_tag}",
            f"    :target: https://github.com/flacjacket/pywayland/actions",
            f"    :alt: Build Status",
            f".. |coveralls| image:: https://coveralls.io/repos/flacjacket/pywayland/badge.svg?branch={version_tag}",
            f"    :target: https://coveralls.io/github/flacjacket/pywayland?branch={version_tag}",
            f"    :alt: Build Coverage",
            f".. |docs| image:: https://readthedocs.org/projects/pywayland/badge/?version={version_tag}",
            f"    :target: https://pywayland.readthedocs.io/en/{version_tag}/",
            f"    :alt: Documentation Status",
        ]
    )

long_description = "\n".join(rst_input)

setup(
    long_description=long_description,
    long_description_content_type="text/x-rst",
    cmdclass=cmdclass,
    cffi_modules=["pywayland/ffi_build.py:ffi_builder"],
)



# This file has been autogenerated by the pywayland scanner

# Copyright © 2018 NXP
# Copyright © 2019 Status Research & Development GmbH.
# Copyright © 2021 Xaver Hugl
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource


class WpDrmLeaseConnectorV1(Interface):
    """A leasable drm connector

    Represents a DRM connector which is available for lease. These objects are
    created via :func:`WpDrmLeaseDeviceV1.connector()
    <pywayland.protocol.drm_lease_v1.WpDrmLeaseDeviceV1.connector>` events, and
    should be passed to lease requests via
    :func:`WpDrmLeaseRequestV1.request_connector()
    <pywayland.protocol.drm_lease_v1.WpDrmLeaseRequestV1.request_connector>`.
    Immediately after the :class:`WpDrmLeaseConnectorV1` object is created the
    compositor will send a name, a description, a connector_id and a done
    event. When the description is updated the compositor will send a
    description event followed by a done event.
    """

    name = "wp_drm_lease_connector_v1"
    version = 1


class WpDrmLeaseConnectorV1Proxy(Proxy):
    interface = WpDrmLeaseConnectorV1

    @WpDrmLeaseConnectorV1.request()
    def destroy(self):
        """Destroy connector

        The client may send this request to indicate that it will not use this
        connector. Clients are encouraged to send this after receiving the
        "withdrawn" event so that the server can release the resources
        associated with this connector offer. Neither existing lease requests
        nor leases will be affected.
        """
        self._marshal(0)
        self._destroy()


class WpDrmLeaseConnectorV1Resource(Resource):
    interface = WpDrmLeaseConnectorV1

    @WpDrmLeaseConnectorV1.event(
        Argument(ArgumentType.String),
    )
    def name(self, name):
        """Name

        The compositor sends this event once the connector is created to
        indicate the name of this connector. This will not change for the
        duration of the Wayland session, but is not guaranteed to be consistent
        between sessions.

        :param name:
            connector name
        :type name:
            `ArgumentType.String`
        """
        self._post_event(0, name)

    @WpDrmLeaseConnectorV1.event(
        Argument(ArgumentType.String),
    )
    def description(self, description):
        """Description

        The compositor sends this event once the connector is created to
        provide a human-readable description for this connector, which may be
        presented to the user. The compositor may send this event multiple
        times over the lifetime of this object to reflect changes in the
        description.

        :param description:
            connector description
        :type description:
            `ArgumentType.String`
        """
        self._post_event(1, description)

    @WpDrmLeaseConnectorV1.event(
        Argument(ArgumentType.Uint),
    )
    def connector_id(self, connector_id):
        """Connector_id

        The compositor sends this event once the connector is created to
        indicate the DRM object ID which represents the underlying connector
        that is being offered. Note that the final lease may include additional
        object IDs, such as CRTCs and planes.

        :param connector_id:
            DRM connector ID
        :type connector_id:
            `ArgumentType.Uint`
        """
        self._post_event(2, connector_id)

    @WpDrmLeaseConnectorV1.event()
    def done(self):
        """All properties have been sent

        This event is sent after all properties of a connector have been sent.
        This allows changes to the properties to be seen as atomic even if they
        happen via multiple events.
        """
        self._post_event(3)

    @WpDrmLeaseConnectorV1.event()
    def withdrawn(self):
        """Lease offer withdrawn

        Sent to indicate that the compositor will no longer honor requests for
        DRM leases which include this connector. The client may still issue a
        lease request including this connector, but the compositor will send
        :func:`WpDrmLeaseV1.finished()
        <pywayland.protocol.drm_lease_v1.WpDrmLeaseV1.finished>` without
        issuing a lease fd. Compositors are encouraged to send this event when
        they lose access to connector, for example when the connector is hot-
        unplugged, when the connector gets leased to a client or when the
        compositor loses DRM master.
        """
        self._post_event(4)


class WpDrmLeaseConnectorV1Global(Global):
    interface = WpDrmLeaseConnectorV1


WpDrmLeaseConnectorV1._gen_c()
WpDrmLeaseConnectorV1.proxy_class = WpDrmLeaseConnectorV1Proxy
WpDrmLeaseConnectorV1.resource_class = WpDrmLeaseConnectorV1Resource
WpDrmLeaseConnectorV1.global_class = WpDrmLeaseConnectorV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2014      Jonas Ådahl
# Copyright © 2015      Red Hat Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlPointer
from ..wayland import WlRegion
from ..wayland import WlSurface
from .zwp_confined_pointer_v1 import ZwpConfinedPointerV1
from .zwp_locked_pointer_v1 import ZwpLockedPointerV1


class ZwpPointerConstraintsV1(Interface):
    """Constrain the movement of a pointer

    The global interface exposing pointer constraining functionality. It
    exposes two requests: lock_pointer for locking the pointer to its position,
    and confine_pointer for locking the pointer to a region.

    The lock_pointer and confine_pointer requests create the objects
    wp_locked_pointer and wp_confined_pointer respectively, and the client can
    use these objects to interact with the lock.

    For any surface, only one lock or confinement may be active across all
    :class:`~pywayland.protocol.wayland.WlPointer` objects of the same seat. If
    a lock or confinement is requested when another lock or confinement is
    active or requested on the same surface and with any of the
    :class:`~pywayland.protocol.wayland.WlPointer` objects of the same seat, an
    'already_constrained' error will be raised.
    """

    name = "zwp_pointer_constraints_v1"
    version = 1

    class error(enum.IntEnum):
        already_constrained = 1

    class lifetime(enum.IntEnum):
        oneshot = 1
        persistent = 2


class ZwpPointerConstraintsV1Proxy(Proxy):
    interface = ZwpPointerConstraintsV1

    @ZwpPointerConstraintsV1.request()
    def destroy(self):
        """Destroy the pointer constraints manager object

        Used by the client to notify the server that it will no longer use this
        pointer constraints object.
        """
        self._marshal(0)
        self._destroy()

    @ZwpPointerConstraintsV1.request(
        Argument(ArgumentType.NewId, interface=ZwpLockedPointerV1),
        Argument(ArgumentType.Object, interface=WlSurface),
        Argument(ArgumentType.Object, interface=WlPointer),
        Argument(ArgumentType.Object, interface=WlRegion, nullable=True),
        Argument(ArgumentType.Uint),
    )
    def lock_pointer(self, surface, pointer, region, lifetime):
        """Lock pointer to a position

        The lock_pointer request lets the client request to disable movements
        of the virtual pointer (i.e. the cursor), effectively locking the
        pointer to a position. This request may not take effect immediately; in
        the future, when the compositor deems implementation-specific
        constraints are satisfied, the pointer lock will be activated and the
        compositor sends a locked event.

        The protocol provides no guarantee that the constraints are ever
        satisfied, and does not require the compositor to send an error if the
        constraints cannot ever be satisfied. It is thus possible to request a
        lock that will never activate.

        There may not be another pointer constraint of any kind requested or
        active on the surface for any of the
        :class:`~pywayland.protocol.wayland.WlPointer` objects of the seat of
        the passed pointer when requesting a lock. If there is, an error will
        be raised. See general pointer lock documentation for more details.

        The intersection of the region passed with this request and the input
        region of the surface is used to determine where the pointer must be in
        order for the lock to activate. It is up to the compositor whether to
        warp the pointer or require some kind of user interaction for the lock
        to activate. If the region is null the surface input region is used.

        A surface may receive pointer focus without the lock being activated.

        The request creates a new object wp_locked_pointer which is used to
        interact with the lock as well as receive updates about its state. See
        the the description of wp_locked_pointer for further information.

        Note that while a pointer is locked, the
        :class:`~pywayland.protocol.wayland.WlPointer` objects of the
        corresponding seat will not emit any :func:`WlPointer.motion()
        <pywayland.protocol.wayland.WlPointer.motion>` events, but relative
        motion events will still be emitted via wp_relative_pointer objects of
        the same seat. :func:`WlPointer.axis()
        <pywayland.protocol.wayland.WlPointer.axis>` and
        :func:`WlPointer.button()
        <pywayland.protocol.wayland.WlPointer.button>` events are unaffected.

        :param surface:
            surface to lock pointer to
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :param pointer:
            the pointer that should be locked
        :type pointer:
            :class:`~pywayland.protocol.wayland.WlPointer`
        :param region:
            region of surface
        :type region:
            :class:`~pywayland.protocol.wayland.WlRegion` or `None`
        :param lifetime:
            lock lifetime
        :type lifetime:
            `ArgumentType.Uint`
        :returns:
            :class:`~pywayland.protocol.pointer_constraints_unstable_v1.ZwpLockedPointerV1`
        """
        id = self._marshal_constructor(1, ZwpLockedPointerV1, surface, pointer, region, lifetime)
        return id

    @ZwpPointerConstraintsV1.request(
        Argument(ArgumentType.NewId, interface=ZwpConfinedPointerV1),
        Argument(ArgumentType.Object, interface=WlSurface),
        Argument(ArgumentType.Object, interface=WlPointer),
        Argument(ArgumentType.Object, interface=WlRegion, nullable=True),
        Argument(ArgumentType.Uint),
    )
    def confine_pointer(self, surface, pointer, region, lifetime):
        """Confine pointer to a region

        The confine_pointer request lets the client request to confine the
        pointer cursor to a given region. This request may not take effect
        immediately; in the future, when the compositor deems implementation-
        specific constraints are satisfied, the pointer confinement will be
        activated and the compositor sends a confined event.

        The intersection of the region passed with this request and the input
        region of the surface is used to determine where the pointer must be in
        order for the confinement to activate. It is up to the compositor
        whether to warp the pointer or require some kind of user interaction
        for the confinement to activate. If the region is null the surface
        input region is used.

        The request will create a new object wp_confined_pointer which is used
        to interact with the confinement as well as receive updates about its
        state. See the the description of wp_confined_pointer for further
        information.

        :param surface:
            surface to lock pointer to
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :param pointer:
            the pointer that should be confined
        :type pointer:
            :class:`~pywayland.protocol.wayland.WlPointer`
        :param region:
            region of surface
        :type region:
            :class:`~pywayland.protocol.wayland.WlRegion` or `None`
        :param lifetime:
            confinement lifetime
        :type lifetime:
            `ArgumentType.Uint`
        :returns:
            :class:`~pywayland.protocol.pointer_constraints_unstable_v1.ZwpConfinedPointerV1`
        """
        id = self._marshal_constructor(2, ZwpConfinedPointerV1, surface, pointer, region, lifetime)
        return id


class ZwpPointerConstraintsV1Resource(Resource):
    interface = ZwpPointerConstraintsV1


class ZwpPointerConstraintsV1Global(Global):
    interface = ZwpPointerConstraintsV1


ZwpPointerConstraintsV1._gen_c()
ZwpPointerConstraintsV1.proxy_class = ZwpPointerConstraintsV1Proxy
ZwpPointerConstraintsV1.resource_class = ZwpPointerConstraintsV1Resource
ZwpPointerConstraintsV1.global_class = ZwpPointerConstraintsV1Global



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import logging
import os
import shlex
import subprocess
from typing import List

from .protocol import Protocol

logger = logging.getLogger(__name__)


def pkgconfig(package, variable) -> str:
    """pkg-config"""
    pkgconfig_env = os.environ.get("PKG_CONFIG", "pkg-config")
    cmd = f"{pkgconfig_env} --variable={variable} {package}"
    args = shlex.split(cmd)
    return subprocess.check_output(args).decode().strip()


def get_wayland_protocols() -> List[str]:
    # use pkg-config to try to find the wayland-protocol directory
    try:
        protocols_dir = pkgconfig("wayland-protocols", "pkgdatadir")
    except subprocess.CalledProcessError:
        raise OSError("Unable to find wayland-protocols using pkgconfig")

    protocols = []
    # walk the wayland-protocol dir
    for dirpath, _, filenames in os.walk(protocols_dir):
        for filename in filenames:
            file_base, file_ext = os.path.splitext(filename)
            # only generate protocols for xml files
            if file_ext == ".xml":
                protocols.append(os.path.join(dirpath, filename))

    # this is pretty brittle, there is an xdg_popup in both the unstable and
    # stable xdg_shell implementations. process the unstable version first so
    # the import is correct when the pop-up tries to import other interfaces.
    return sorted(protocols, reverse=True)


def main() -> None:
    this_dir = os.path.split(__file__)[0]
    protocol_dir = os.path.join(this_dir, "..", "protocol")

    # try to figure out where the wayland.xml file is installed, otherwise use
    # default
    try:
        xml_dir = pkgconfig("wayland-scanner", "pkgdatadir")
        xml_file = os.path.join(xml_dir, "wayland.xml")
    except subprocess.CalledProcessError:
        xml_file = "/usr/share/wayland/wayland.xml"

    parser = argparse.ArgumentParser(
        description="Generate wayland protocol files from xml"
    )
    parser.add_argument(
        "-o",
        "--output-dir",
        metavar="DIR",
        default=protocol_dir,
        type=str,
        help="Directory to output protocol files",
    )
    parser.add_argument(
        "--with-protocols",
        action="store_true",
        help="Also locate and build wayland-protocol xml files (using pkg-config)",
    )
    parser.add_argument(
        "-i",
        "--input",
        metavar="XML_FILE",
        default=[xml_file],
        nargs="+",
        type=str,
        help="Path to input xml file to scan",
    )

    args = parser.parse_args()

    if not os.path.exists(args.output_dir):
        os.makedirs(args.output_dir, 0o775)

    input_files = args.input

    if args.with_protocols:
        protocols_files = get_wayland_protocols()
        input_files += protocols_files

    protocols = [Protocol.parse_file(input_file) for input_file in input_files]
    logger.info("Parsed {} input xml files".format(len(protocols)))

    protocol_imports = {
        interface.name: protocol.name
        for protocol in protocols
        for interface in protocol.interface
    }

    for protocol in protocols:
        protocol.output(args.output_dir, protocol_imports)
        logger.info("Generated protocol: {}".format(protocol.name))


if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)
    main()



# This file has been autogenerated by the pywayland scanner

# Copyright © 2017 Red Hat Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Global, Interface, Proxy, Resource


class ZwpKeyboardShortcutsInhibitorV1(Interface):
    """Context object for keyboard shortcuts inhibitor

    A keyboard shortcuts inhibitor instructs the compositor to ignore its own
    keyboard shortcuts when the associated surface has keyboard focus. As a
    result, when the surface has keyboard focus on the given seat, it will
    receive all key events originating from the specified seat, even those
    which would normally be caught by the compositor for its own shortcuts.

    The Wayland compositor is however under no obligation to disable all of its
    shortcuts, and may keep some special key combo for its own use, including
    but not limited to one allowing the user to forcibly restore normal
    keyboard events routing in the case of an unwilling client. The compositor
    may also use the same key combo to reactivate an existing shortcut
    inhibitor that was previously deactivated on user request.

    When the compositor restores its own keyboard shortcuts, an "inactive"
    event is emitted to notify the client that the keyboard shortcuts inhibitor
    is not effectively active for the surface and seat any more, and the client
    should not expect to receive all keyboard events.

    When the keyboard shortcuts inhibitor is inactive, the client has no way to
    forcibly reactivate the keyboard shortcuts inhibitor.

    The user can chose to re-enable a previously deactivated keyboard shortcuts
    inhibitor using any mechanism the compositor may offer, in which case the
    compositor will send an "active" event to notify the client.

    If the surface is destroyed, unmapped, or loses the seat's keyboard focus,
    the keyboard shortcuts inhibitor becomes irrelevant and the compositor will
    restore its own keyboard shortcuts but no "inactive" event is emitted in
    this case.
    """

    name = "zwp_keyboard_shortcuts_inhibitor_v1"
    version = 1


class ZwpKeyboardShortcutsInhibitorV1Proxy(Proxy):
    interface = ZwpKeyboardShortcutsInhibitorV1

    @ZwpKeyboardShortcutsInhibitorV1.request()
    def destroy(self):
        """Destroy the keyboard shortcuts inhibitor object

        Remove the keyboard shortcuts inhibitor from the associated
        :class:`~pywayland.protocol.wayland.WlSurface`.
        """
        self._marshal(0)
        self._destroy()


class ZwpKeyboardShortcutsInhibitorV1Resource(Resource):
    interface = ZwpKeyboardShortcutsInhibitorV1

    @ZwpKeyboardShortcutsInhibitorV1.event()
    def active(self):
        """Shortcuts are inhibited

        This event indicates that the shortcut inhibitor is active.

        The compositor sends this event every time compositor shortcuts are
        inhibited on behalf of the surface. When active, the client may receive
        input events normally reserved by the compositor (see
        :class:`ZwpKeyboardShortcutsInhibitorV1`).

        This occurs typically when the initial request "inhibit_shortcuts"
        first becomes active or when the user instructs the compositor to re-
        enable and existing shortcuts inhibitor using any mechanism offered by
        the compositor.
        """
        self._post_event(0)

    @ZwpKeyboardShortcutsInhibitorV1.event()
    def inactive(self):
        """Shortcuts are restored

        This event indicates that the shortcuts inhibitor is inactive, normal
        shortcuts processing is restored by the compositor.
        """
        self._post_event(1)


class ZwpKeyboardShortcutsInhibitorV1Global(Global):
    interface = ZwpKeyboardShortcutsInhibitorV1


ZwpKeyboardShortcutsInhibitorV1._gen_c()
ZwpKeyboardShortcutsInhibitorV1.proxy_class = ZwpKeyboardShortcutsInhibitorV1Proxy
ZwpKeyboardShortcutsInhibitorV1.resource_class = ZwpKeyboardShortcutsInhibitorV1Resource
ZwpKeyboardShortcutsInhibitorV1.global_class = ZwpKeyboardShortcutsInhibitorV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from .wl_shm_pool import WlShmPool


class WlShm(Interface):
    """Shared memory support

    A singleton global object that provides support for shared memory.

    Clients can create :class:`~pywayland.protocol.wayland.WlShmPool` objects
    using the create_pool request.

    At connection setup time, the :class:`WlShm` object emits one or more
    format events to inform clients about the valid pixel formats that can be
    used for buffers.
    """

    name = "wl_shm"
    version = 1

    class error(enum.IntEnum):
        invalid_format = 0
        invalid_stride = 1
        invalid_fd = 2

    class format(enum.IntEnum):
        argb8888 = 0
        xrgb8888 = 1
        c8 = 0x20203843
        rgb332 = 0x38424752
        bgr233 = 0x38524742
        xrgb4444 = 0x32315258
        xbgr4444 = 0x32314258
        rgbx4444 = 0x32315852
        bgrx4444 = 0x32315842
        argb4444 = 0x32315241
        abgr4444 = 0x32314241
        rgba4444 = 0x32314152
        bgra4444 = 0x32314142
        xrgb1555 = 0x35315258
        xbgr1555 = 0x35314258
        rgbx5551 = 0x35315852
        bgrx5551 = 0x35315842
        argb1555 = 0x35315241
        abgr1555 = 0x35314241
        rgba5551 = 0x35314152
        bgra5551 = 0x35314142
        rgb565 = 0x36314752
        bgr565 = 0x36314742
        rgb888 = 0x34324752
        bgr888 = 0x34324742
        xbgr8888 = 0x34324258
        rgbx8888 = 0x34325852
        bgrx8888 = 0x34325842
        abgr8888 = 0x34324241
        rgba8888 = 0x34324152
        bgra8888 = 0x34324142
        xrgb2101010 = 0x30335258
        xbgr2101010 = 0x30334258
        rgbx1010102 = 0x30335852
        bgrx1010102 = 0x30335842
        argb2101010 = 0x30335241
        abgr2101010 = 0x30334241
        rgba1010102 = 0x30334152
        bgra1010102 = 0x30334142
        yuyv = 0x56595559
        yvyu = 0x55595659
        uyvy = 0x59565955
        vyuy = 0x59555956
        ayuv = 0x56555941
        nv12 = 0x3231564E
        nv21 = 0x3132564E
        nv16 = 0x3631564E
        nv61 = 0x3136564E
        yuv410 = 0x39565559
        yvu410 = 0x39555659
        yuv411 = 0x31315559
        yvu411 = 0x31315659
        yuv420 = 0x32315559
        yvu420 = 0x32315659
        yuv422 = 0x36315559
        yvu422 = 0x36315659
        yuv444 = 0x34325559
        yvu444 = 0x34325659
        r8 = 0x20203852
        r16 = 0x20363152
        rg88 = 0x38384752
        gr88 = 0x38385247
        rg1616 = 0x32334752
        gr1616 = 0x32335247
        xrgb16161616f = 0x48345258
        xbgr16161616f = 0x48344258
        argb16161616f = 0x48345241
        abgr16161616f = 0x48344241
        xyuv8888 = 0x56555958
        vuy888 = 0x34325556
        vuy101010 = 0x30335556
        y210 = 0x30313259
        y212 = 0x32313259
        y216 = 0x36313259
        y410 = 0x30313459
        y412 = 0x32313459
        y416 = 0x36313459
        xvyu2101010 = 0x30335658
        xvyu12_16161616 = 0x36335658
        xvyu16161616 = 0x38345658
        y0l0 = 0x304C3059
        x0l0 = 0x304C3058
        y0l2 = 0x324C3059
        x0l2 = 0x324C3058
        yuv420_8bit = 0x38305559
        yuv420_10bit = 0x30315559
        xrgb8888_a8 = 0x38415258
        xbgr8888_a8 = 0x38414258
        rgbx8888_a8 = 0x38415852
        bgrx8888_a8 = 0x38415842
        rgb888_a8 = 0x38413852
        bgr888_a8 = 0x38413842
        rgb565_a8 = 0x38413552
        bgr565_a8 = 0x38413542
        nv24 = 0x3432564E
        nv42 = 0x3234564E
        p210 = 0x30313250
        p010 = 0x30313050
        p012 = 0x32313050
        p016 = 0x36313050
        axbxgxrx106106106106 = 0x30314241
        nv15 = 0x3531564E
        q410 = 0x30313451
        q401 = 0x31303451


class WlShmProxy(Proxy):
    interface = WlShm

    @WlShm.request(
        Argument(ArgumentType.NewId, interface=WlShmPool),
        Argument(ArgumentType.FileDescriptor),
        Argument(ArgumentType.Int),
    )
    def create_pool(self, fd, size):
        """Create a shm pool

        Create a new :class:`~pywayland.protocol.wayland.WlShmPool` object.

        The pool can be used to create shared memory based buffer objects.  The
        server will mmap size bytes of the passed file descriptor, to use as
        backing memory for the pool.

        :param fd:
            file descriptor for the pool
        :type fd:
            `ArgumentType.FileDescriptor`
        :param size:
            pool size, in bytes
        :type size:
            `ArgumentType.Int`
        :returns:
            :class:`~pywayland.protocol.wayland.WlShmPool` -- pool to create
        """
        id = self._marshal_constructor(0, WlShmPool, fd, size)
        return id


class WlShmResource(Resource):
    interface = WlShm

    @WlShm.event(
        Argument(ArgumentType.Uint),
    )
    def format(self, format):
        """Pixel format description

        Informs the client about a valid pixel format that can be used for
        buffers. Known formats include argb8888 and xrgb8888.

        :param format:
            buffer pixel format
        :type format:
            `ArgumentType.Uint`
        """
        self._post_event(0, format)


class WlShmGlobal(Global):
    interface = WlShm


WlShm._gen_c()
WlShm.proxy_class = WlShmProxy
WlShm.resource_class = WlShmResource
WlShm.global_class = WlShmGlobal



# This file has been autogenerated by the pywayland scanner

# Copyright © 2017 Red Hat Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlSeat
from ..wayland import WlSurface
from .zwp_keyboard_shortcuts_inhibitor_v1 import ZwpKeyboardShortcutsInhibitorV1


class ZwpKeyboardShortcutsInhibitManagerV1(Interface):
    """Context object for keyboard grab_manager

    A global interface used for inhibiting the compositor keyboard shortcuts.
    """

    name = "zwp_keyboard_shortcuts_inhibit_manager_v1"
    version = 1

    class error(enum.IntEnum):
        already_inhibited = 0


class ZwpKeyboardShortcutsInhibitManagerV1Proxy(Proxy):
    interface = ZwpKeyboardShortcutsInhibitManagerV1

    @ZwpKeyboardShortcutsInhibitManagerV1.request()
    def destroy(self):
        """Destroy the keyboard shortcuts inhibitor object

        Destroy the keyboard shortcuts inhibitor manager.
        """
        self._marshal(0)
        self._destroy()

    @ZwpKeyboardShortcutsInhibitManagerV1.request(
        Argument(ArgumentType.NewId, interface=ZwpKeyboardShortcutsInhibitorV1),
        Argument(ArgumentType.Object, interface=WlSurface),
        Argument(ArgumentType.Object, interface=WlSeat),
    )
    def inhibit_shortcuts(self, surface, seat):
        """Create a new keyboard shortcuts inhibitor object

        Create a new keyboard shortcuts inhibitor object associated with the
        given surface for the given seat.

        If shortcuts are already inhibited for the specified seat and surface,
        a protocol error "already_inhibited" is raised by the compositor.

        :param surface:
            the surface that inhibits the keyboard shortcuts behavior
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :param seat:
            the :class:`~pywayland.protocol.wayland.WlSeat` for which keyboard
            shortcuts should be disabled
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        :returns:
            :class:`~pywayland.protocol.keyboard_shortcuts_inhibit_unstable_v1.ZwpKeyboardShortcutsInhibitorV1`
        """
        id = self._marshal_constructor(1, ZwpKeyboardShortcutsInhibitorV1, surface, seat)
        return id


class ZwpKeyboardShortcutsInhibitManagerV1Resource(Resource):
    interface = ZwpKeyboardShortcutsInhibitManagerV1


class ZwpKeyboardShortcutsInhibitManagerV1Global(Global):
    interface = ZwpKeyboardShortcutsInhibitManagerV1


ZwpKeyboardShortcutsInhibitManagerV1._gen_c()
ZwpKeyboardShortcutsInhibitManagerV1.proxy_class = ZwpKeyboardShortcutsInhibitManagerV1Proxy
ZwpKeyboardShortcutsInhibitManagerV1.resource_class = ZwpKeyboardShortcutsInhibitManagerV1Resource
ZwpKeyboardShortcutsInhibitManagerV1.global_class = ZwpKeyboardShortcutsInhibitManagerV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2012, 2013 Intel Corporation
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlOutput


class ZwpInputPanelSurfaceV1(Interface):
    name = "zwp_input_panel_surface_v1"
    version = 1

    class position(enum.IntEnum):
        center_bottom = 0


class ZwpInputPanelSurfaceV1Proxy(Proxy):
    interface = ZwpInputPanelSurfaceV1

    @ZwpInputPanelSurfaceV1.request(
        Argument(ArgumentType.Object, interface=WlOutput),
        Argument(ArgumentType.Uint),
    )
    def set_toplevel(self, output, position):
        """Set the surface type as a keyboard

        Set the input_panel_surface type to keyboard.

        A keyboard surface is only shown when a text input is active.

        :param output:
        :type output:
            :class:`~pywayland.protocol.wayland.WlOutput`
        :param position:
        :type position:
            `ArgumentType.Uint`
        """
        self._marshal(0, output, position)

    @ZwpInputPanelSurfaceV1.request()
    def set_overlay_panel(self):
        """Set the surface type as an overlay panel

        Set the input_panel_surface to be an overlay panel.

        This is shown near the input cursor above the application window when a
        text input is active.
        """
        self._marshal(1)


class ZwpInputPanelSurfaceV1Resource(Resource):
    interface = ZwpInputPanelSurfaceV1


class ZwpInputPanelSurfaceV1Global(Global):
    interface = ZwpInputPanelSurfaceV1


ZwpInputPanelSurfaceV1._gen_c()
ZwpInputPanelSurfaceV1.proxy_class = ZwpInputPanelSurfaceV1Proxy
ZwpInputPanelSurfaceV1.resource_class = ZwpInputPanelSurfaceV1Resource
ZwpInputPanelSurfaceV1.global_class = ZwpInputPanelSurfaceV1Global



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pywayland.server import Client, Listener, Display
from pywayland.protocol.wayland import WlDisplay

import socket


def test_create_resource():
    s1, s2 = socket.socketpair(socket.AF_UNIX, socket.SOCK_STREAM, 0)
    display = Display()
    client = Client(display, s1.fileno())

    # Create resource
    res = WlDisplay.resource_class(client, version=4)

    assert res.version == 4

    # Fetching the client object by id gives the resource back again
    assert client.get_object(res.id) == res

    client.user_data = 0xbee
    assert client.user_data == 0xbee

    client.destroy()
    display.destroy()

    s2.close()


def _destroy_callback(data):
    global destroyed
    destroyed = True


def _destroy_notify(listener, data):
    global notified
    notified = True


def test_destroy_resource():
    global destroyed, notified
    destroyed = False
    notified = False

    s1, s2 = socket.socketpair(socket.AF_UNIX, socket.SOCK_STREAM, 0)
    display = Display()
    client = Client(display, s1.fileno())
    listener = Listener(_destroy_notify)

    # Create resource
    res = WlDisplay.resource_class(client, version=4)
    # Attach a destructor and a destroy notification
    res.dispatcher.destructor = _destroy_callback
    res.add_destroy_listener(listener)
    # Destroy the resource
    res.destroy()

    assert destroyed
    assert notified

    assert client.get_object(res.id) is None

    # Create resource
    res = WlDisplay.resource_class(client, version=2)
    # Attach a destructor and a destroy notification
    res.dispatcher.destructor = _destroy_callback
    res.add_destroy_listener(listener)
    # Destroy the client
    client.destroy()

    assert destroyed
    assert notified

    display.destroy()

    s2.close()


def notest_create_resource_with_same_id():
    s1, s2 = socket.socketpair(socket.AF_UNIX, socket.SOCK_STREAM, 0)
    display = Display()
    client = Client(display, s1.fileno())

    # Create resource
    res = WlDisplay.resource_class(client, version=2)
    assert client.get_object(res.id) == res

    # This should replace the old one
    res2 = WlDisplay.resource_class(client, version=1, id=res.id)
    assert client.get_object(res.id) == res2

    res2.destroy()
    res.destroy()

    client.destroy()
    display.destroy()

    s2.close()



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import functools
from weakref import WeakKeyDictionary

from pywayland import ffi, lib

weakkeydict: WeakKeyDictionary = WeakKeyDictionary()


# void (*wl_global_bind_func_t)(struct wl_client *client, void *data, uint32_t version, uint32_t id)
@ffi.def_extern()
def global_bind_func(client_ptr, data, version, id) -> None:
    # `data` is the handle to Global
    callback_info = ffi.from_handle(data)

    version = min(callback_info["interface"].version, version)
    resource = callback_info["interface"].resource_class(client_ptr, version, id)

    # Call a user defined handler
    if callback_info["bind_func"]:
        # TODO: add some error catching so we don't segfault
        callback_info["bind_func"](resource)


def _global_destroy(display, cdata) -> None:
    if display._ptr is not None:
        # TODO: figure out how this can get run...
        # lib.wl_global_destroy(cdata)
        pass


class Global:
    """A server-side Interface object for the server

    Not created directly, created from the
    :class:`~pywayland.interface.Interface` object.

    :param display: The display the object is created on
    :type display: :class:`~pywayland.server.Display`
    :param version: The version to use for the
                    :class:`~pywayland.interface.Interface`, uses current
                    version if not specified
    :type version: `int`
    """

    def __init__(self, display, version=None):
        if display._ptr is None or display._ptr == ffi.NULL:
            raise ValueError("Display has been destroyed or couldn't initialize")

        if version is None:
            version = self.interface.version

        # we can't keep alive a handle to self without creating a reference
        # loop, so use this dict as the handle to pass to the global_bind_func
        # callback
        self._callback_info = {"interface": self.interface, "bind_func": None}
        self._handle = ffi.new_handle(self._callback_info)

        ptr = lib.wl_global_create(
            display._ptr,
            self.interface._ptr,
            version,
            self._handle,
            lib.global_bind_func,
        )
        destructor = functools.partial(_global_destroy, display)
        self._ptr = ffi.gc(ptr, destructor)
        self._display = display

        # this c data should keep the display alive
        weakkeydict[self._ptr] = display

    @property
    def bind_func(self):
        return self._callback_info["bind_func"]

    @bind_func.setter
    def bind_func(self, value):
        self._callback_info["bind_func"] = value

    def destroy(self):
        """Destroy the global object"""
        if self._ptr is not None:
            # run and remove destructor on c data
            _global_destroy(self._display, self._ptr)
            ffi.gc(self._ptr, None)
            self._ptr = None
            self._display = None
